/*
 * Copyright 2002 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * @(#)AccessBridgeCalls.h	1.13 03/09/02
 */

/* 
 * Wrapper functions around calls to the AccessBridge DLL
 */

#include <windows.h>
#include <jni.h>
#include "AccessBridgeCallbacks.h"
#include "AccessBridgePackages.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef jobject AccessibleContext;
typedef jobject AccessibleText;
typedef jobject AccessibleValue;
typedef jobject AccessibleSelection;
typedef jobject Java_Object;
typedef jobject PropertyChangeEvent;
typedef jobject FocusEvent;
typedef jobject CaretEvent;
typedef jobject MouseEvent;
typedef jobject MenuEvent;
typedef jobject AccessibleTable;


typedef void (*Windows_runFP) ();

typedef void (*SetPropertyChangeFP) (AccessBridge_PropertyChangeFP fp);

typedef void (*SetFocusGainedFP) (AccessBridge_FocusGainedFP fp);
typedef void (*SetFocusLostFP) (AccessBridge_FocusLostFP fp);

typedef void (*SetCaretUpdateFP) (AccessBridge_CaretUpdateFP fp);

typedef void (*SetMouseClickedFP) (AccessBridge_MouseClickedFP fp);
typedef void (*SetMouseEnteredFP) (AccessBridge_MouseEnteredFP fp);
typedef void (*SetMouseExitedFP) (AccessBridge_MouseExitedFP fp);
typedef void (*SetMousePressedFP) (AccessBridge_MousePressedFP fp);
typedef void (*SetMouseReleasedFP) (AccessBridge_MouseReleasedFP fp);

typedef void (*SetMenuCanceledFP) (AccessBridge_MenuCanceledFP fp);
typedef void (*SetMenuDeselectedFP) (AccessBridge_MenuDeselectedFP fp);
typedef void (*SetMenuSelectedFP) (AccessBridge_MenuSelectedFP fp);
typedef void (*SetPopupMenuCanceledFP) (AccessBridge_PopupMenuCanceledFP fp);
typedef void (*SetPopupMenuWillBecomeInvisibleFP) (AccessBridge_PopupMenuWillBecomeInvisibleFP fp);
typedef void (*SetPopupMenuWillBecomeVisibleFP) (AccessBridge_PopupMenuWillBecomeVisibleFP fp);

typedef void (*SetPropertyNameChangeFP) (AccessBridge_PropertyNameChangeFP fp);
typedef void (*SetPropertyDescriptionChangeFP) (AccessBridge_PropertyDescriptionChangeFP fp);
typedef void (*SetPropertyStateChangeFP) (AccessBridge_PropertyStateChangeFP fp);
typedef void (*SetPropertyValueChangeFP) (AccessBridge_PropertyValueChangeFP fp);
typedef void (*SetPropertySelectionChangeFP) (AccessBridge_PropertySelectionChangeFP fp);
typedef void (*SetPropertyTextChangeFP) (AccessBridge_PropertyTextChangeFP fp);
typedef void (*SetPropertyCaretChangeFP) (AccessBridge_PropertyCaretChangeFP fp);
typedef void (*SetPropertyVisibleDataChangeFP) (AccessBridge_PropertyVisibleDataChangeFP fp);
typedef void (*SetPropertyChildChangeFP) (AccessBridge_PropertyChildChangeFP fp);
typedef void (*SetPropertyActiveDescendentChangeFP) (AccessBridge_PropertyActiveDescendentChangeFP fp);

typedef void (*SetPropertyTableModelChangeFP) (AccessBridge_PropertyTableModelChangeFP fp);

typedef void (*ReleaseJavaObjectFP) (long vmID, Java_Object object);

typedef BOOL (*GetVersionInfoFP) (long vmID, AccessBridgeVersionInfo *info);

typedef BOOL (*IsJavaWindowFP) (HWND window);
typedef BOOL (*IsSameObjectFP) (long vmID, jobject obj1, jobject obj2);
typedef BOOL (*GetAccessibleContextFromHWNDFP) (HWND window, long *vmID, AccessibleContext *ac);

typedef BOOL (*GetAccessibleContextAtFP) (long vmID, AccessibleContext acParent, 
										  jint x, jint y, AccessibleContext *ac);
typedef BOOL (*GetAccessibleContextWithFocusFP) (HWND window, long *vmID, AccessibleContext *ac);
typedef BOOL (*GetAccessibleContextInfoFP) (long vmID, AccessibleContext ac, AccessibleContextInfo *info);
typedef AccessibleContext (*GetAccessibleChildFromContextFP) (long vmID, AccessibleContext ac, jint i);
typedef AccessibleContext (*GetAccessibleParentFromContextFP) (long vmID, AccessibleContext ac);

/* begin AccessibleTable */
typedef BOOL (*getAccessibleTableInfoFP) (long vmID, AccessibleContext ac, AccessibleTableInfo *tableInfo);
typedef BOOL (*getAccessibleTableCellInfoFP) (long vmID, AccessibleTable accessibleTable, 
										      jint row, jint column, AccessibleTableCellInfo *tableCellInfo);

typedef BOOL (*getAccessibleTableRowHeaderFP) (long vmID, AccessibleContext acParent, AccessibleTableInfo *tableInfo);
typedef BOOL (*getAccessibleTableColumnHeaderFP) (long vmID, AccessibleContext acParent, AccessibleTableInfo *tableInfo);

typedef AccessibleContext (*getAccessibleTableRowDescriptionFP) (long vmID, AccessibleContext acParent, jint row);
typedef AccessibleContext (*getAccessibleTableColumnDescriptionFP) (long vmID, AccessibleContext acParent, jint column);

typedef jint (*getAccessibleTableRowSelectionCountFP) (long vmID, AccessibleTable table);
typedef BOOL (*isAccessibleTableRowSelectedFP) (long vmID, AccessibleTable table, jint row);
typedef BOOL (*getAccessibleTableRowSelectionsFP) (long vmID, AccessibleTable table, jint count, 
												   jint *selections);

typedef jint (*getAccessibleTableColumnSelectionCountFP) (long vmID, AccessibleTable table);
typedef BOOL (*isAccessibleTableColumnSelectedFP) (long vmID, AccessibleTable table, jint column);
typedef BOOL (*getAccessibleTableColumnSelectionsFP) (long vmID, AccessibleTable table, jint count, 
													  jint *selections);

typedef jint (*getAccessibleTableRowFP) (long vmID, AccessibleTable table, jint index);
typedef jint (*getAccessibleTableColumnFP) (long vmID, AccessibleTable table, jint index);
typedef jint (*getAccessibleTableIndexFP) (long vmID, AccessibleTable table, jint row, jint column);
/* end AccessibleTable */

typedef BOOL (*getAccessibleRelationSetFP) (long vmID, AccessibleContext accessibleContext,
											AccessibleRelationSetInfo *relationSetInfo);

/* AccessibleText */

typedef BOOL (*GetAccessibleTextInfoFP) (long vmID, AccessibleText at, AccessibleTextInfo *textInfo, jint x, jint y);
typedef BOOL (*GetAccessibleTextItemsFP) (long vmID, AccessibleText at, AccessibleTextItemsInfo *textItems, jint index);
typedef BOOL (*GetAccessibleTextSelectionInfoFP) (long vmID, AccessibleText at, AccessibleTextSelectionInfo *textSelection);
typedef BOOL (*GetAccessibleTextAttributesFP) (long vmID, AccessibleText at, jint index, AccessibleTextAttributesInfo *attributes);
typedef BOOL (*GetAccessibleTextRectFP) (long vmID, AccessibleText at, AccessibleTextRectInfo *rectInfo, jint index);
typedef BOOL (*GetAccessibleTextLineBoundsFP) (long vmID, AccessibleText at, jint index, jint *startIndex, jint *endIndex);
typedef BOOL (*GetAccessibleTextRangeFP) (long vmID, AccessibleText at, jint start, jint end, wchar_t *text, short len);

typedef BOOL (*GetCurrentAccessibleValueFromContextFP) (long vmID, AccessibleValue av, wchar_t *value, short len);
typedef BOOL (*GetMaximumAccessibleValueFromContextFP) (long vmID, AccessibleValue av, wchar_t *value, short len);
typedef BOOL (*GetMinimumAccessibleValueFromContextFP) (long vmID, AccessibleValue av, wchar_t *value, short len);

typedef void (*AddAccessibleSelectionFromContextFP) (long vmID, AccessibleSelection as, int i);
typedef void (*ClearAccessibleSelectionFromContextFP) (long vmID, AccessibleSelection as);
typedef jobject (*GetAccessibleSelectionFromContextFP) (long vmID, AccessibleSelection as, int i);
typedef int (*GetAccessibleSelectionCountFromContextFP) (long vmID, AccessibleSelection as);
typedef BOOL (*IsAccessibleChildSelectedFromContextFP) (long vmID, AccessibleSelection as, int i);
typedef void (*RemoveAccessibleSelectionFromContextFP) (long vmID, AccessibleSelection as, int i);
typedef void (*SelectAllAccessibleSelectionFromContextFP) (long vmID, AccessibleSelection as);


typedef struct AccessBridgeFPsTag {
	Windows_runFP Windows_run;

	SetPropertyChangeFP SetPropertyChange;

	SetFocusGainedFP SetFocusGained;
	SetFocusLostFP SetFocusLost;
	
	SetCaretUpdateFP SetCaretUpdate;

	SetMouseClickedFP SetMouseClicked;
    SetMouseEnteredFP SetMouseEntered;
    SetMouseExitedFP SetMouseExited;
    SetMousePressedFP SetMousePressed;
    SetMouseReleasedFP SetMouseReleased;

    SetMenuCanceledFP SetMenuCanceled;
    SetMenuDeselectedFP SetMenuDeselected;
    SetMenuSelectedFP SetMenuSelected;
    SetPopupMenuCanceledFP SetPopupMenuCanceled;
    SetPopupMenuWillBecomeInvisibleFP SetPopupMenuWillBecomeInvisible;
    SetPopupMenuWillBecomeVisibleFP SetPopupMenuWillBecomeVisible;

    SetPropertyNameChangeFP SetPropertyNameChange;
    SetPropertyDescriptionChangeFP SetPropertyDescriptionChange;
    SetPropertyStateChangeFP SetPropertyStateChange;
    SetPropertyValueChangeFP SetPropertyValueChange;
    SetPropertySelectionChangeFP SetPropertySelectionChange;
    SetPropertyTextChangeFP SetPropertyTextChange;
    SetPropertyCaretChangeFP SetPropertyCaretChange;
    SetPropertyVisibleDataChangeFP SetPropertyVisibleDataChange;
    SetPropertyChildChangeFP SetPropertyChildChange;
    SetPropertyActiveDescendentChangeFP SetPropertyActiveDescendentChange;

    SetPropertyTableModelChangeFP SetPropertyTableModelChange;

	ReleaseJavaObjectFP ReleaseJavaObject;
    GetVersionInfoFP GetVersionInfo;

	IsJavaWindowFP IsJavaWindow;
	IsSameObjectFP IsSameObject;
    GetAccessibleContextFromHWNDFP GetAccessibleContextFromHWND;
    
	GetAccessibleContextAtFP GetAccessibleContextAt;
	GetAccessibleContextWithFocusFP GetAccessibleContextWithFocus;
	GetAccessibleContextInfoFP GetAccessibleContextInfo;
	GetAccessibleChildFromContextFP GetAccessibleChildFromContext;
	GetAccessibleParentFromContextFP GetAccessibleParentFromContext;

	getAccessibleTableInfoFP getAccessibleTableInfo;
	getAccessibleTableCellInfoFP getAccessibleTableCellInfo;

	getAccessibleTableRowHeaderFP getAccessibleTableRowHeader;
	getAccessibleTableColumnHeaderFP getAccessibleTableColumnHeader;

	getAccessibleTableRowDescriptionFP getAccessibleTableRowDescription;
	getAccessibleTableColumnDescriptionFP getAccessibleTableColumnDescription;

	getAccessibleTableRowSelectionCountFP getAccessibleTableRowSelectionCount;
    isAccessibleTableRowSelectedFP isAccessibleTableRowSelected;
    getAccessibleTableRowSelectionsFP getAccessibleTableRowSelections;

    getAccessibleTableColumnSelectionCountFP getAccessibleTableColumnSelectionCount;
    isAccessibleTableColumnSelectedFP isAccessibleTableColumnSelected;
    getAccessibleTableColumnSelectionsFP getAccessibleTableColumnSelections;

	getAccessibleTableRowFP getAccessibleTableRow;
	getAccessibleTableColumnFP getAccessibleTableColumn;
	getAccessibleTableIndexFP getAccessibleTableIndex;

	getAccessibleRelationSetFP getAccessibleRelationSet;


	GetAccessibleTextInfoFP GetAccessibleTextInfo;
	GetAccessibleTextItemsFP GetAccessibleTextItems;
	GetAccessibleTextSelectionInfoFP GetAccessibleTextSelectionInfo;
	GetAccessibleTextAttributesFP GetAccessibleTextAttributes;
	GetAccessibleTextRectFP GetAccessibleTextRect;
	GetAccessibleTextLineBoundsFP GetAccessibleTextLineBounds;
	GetAccessibleTextRangeFP GetAccessibleTextRange;

    GetCurrentAccessibleValueFromContextFP GetCurrentAccessibleValueFromContext;
    GetMaximumAccessibleValueFromContextFP GetMaximumAccessibleValueFromContext;
    GetMinimumAccessibleValueFromContextFP GetMinimumAccessibleValueFromContext;
    
    AddAccessibleSelectionFromContextFP AddAccessibleSelectionFromContext;
    ClearAccessibleSelectionFromContextFP ClearAccessibleSelectionFromContext;
    GetAccessibleSelectionFromContextFP GetAccessibleSelectionFromContext;
    GetAccessibleSelectionCountFromContextFP GetAccessibleSelectionCountFromContext;
    IsAccessibleChildSelectedFromContextFP IsAccessibleChildSelectedFromContext;
    RemoveAccessibleSelectionFromContextFP RemoveAccessibleSelectionFromContext;
    SelectAllAccessibleSelectionFromContextFP SelectAllAccessibleSelectionFromContext;

} AccessBridgeFPs;



/**
 * Initialize the world
 */
BOOL initializeAccessBridge();
BOOL shutdownAccessBridge();

/**
 * Window routines
 */
BOOL IsJavaWindow(HWND window);
BOOL GetAccessibleContextFromHWND(HWND target, long *vmID, AccessibleContext *ac);
BOOL IsSameObject(long vmID, jobject obj1, jobject obj2);

/**
 * Event handling routines
 */
//void SetPropertyChange(AccessBridge_PropertyChangeFP fp);     [[[FIXME]]] - remove this

void SetFocusGained(AccessBridge_FocusGainedFP fp);
void SetFocusLost(AccessBridge_FocusLostFP fp);

void SetCaretUpdate(AccessBridge_CaretUpdateFP fp);

void SetMouseClicked(AccessBridge_MouseClickedFP fp);
void SetMouseEntered(AccessBridge_MouseEnteredFP fp);
void SetMouseExited(AccessBridge_MouseExitedFP fp);
void SetMousePressed(AccessBridge_MousePressedFP fp);
void SetMouseReleased(AccessBridge_MouseReleasedFP fp);

void SetMenuCanceled(AccessBridge_MenuCanceledFP fp);
void SetMenuDeselected(AccessBridge_MenuDeselectedFP fp);
void SetMenuSelected(AccessBridge_MenuSelectedFP fp);
void SetPopupMenuCanceled(AccessBridge_PopupMenuCanceledFP fp);
void SetPopupMenuWillBecomeInvisible(AccessBridge_PopupMenuWillBecomeInvisibleFP fp);
void SetPopupMenuWillBecomeVisible(AccessBridge_PopupMenuWillBecomeVisibleFP fp);

void SetPropertyNameChange(AccessBridge_PropertyNameChangeFP fp);
void SetPropertyDescriptionChange(AccessBridge_PropertyDescriptionChangeFP fp);
void SetPropertyStateChange(AccessBridge_PropertyStateChangeFP fp);
void SetPropertyValueChange(AccessBridge_PropertyValueChangeFP fp);
void SetPropertySelectionChange(AccessBridge_PropertySelectionChangeFP fp);
void SetPropertyTextChange(AccessBridge_PropertyTextChangeFP fp);
void SetPropertyCaretChange(AccessBridge_PropertyCaretChangeFP fp);
void SetPropertyVisibleDataChange(AccessBridge_PropertyVisibleDataChangeFP fp);
void SetPropertyChildChange(AccessBridge_PropertyChildChangeFP fp);
void SetPropertyActiveDescendentChange(AccessBridge_PropertyActiveDescendentChangeFP fp);

void SetPropertyTableModelChange(AccessBridge_PropertyTableModelChangeFP fp);

/**
 * General routines
 */
void ReleaseJavaObject(long vmID, Java_Object object);
BOOL GetVersionInfo(long vmID, AccessBridgeVersionInfo *info);

/**
 * Accessible Context routines
 */
BOOL GetAccessibleContextAt(long vmID, AccessibleContext acParent, 
							jint x, jint y, AccessibleContext *ac);
BOOL GetAccessibleContextWithFocus(HWND window, long *vmID, AccessibleContext *ac);
BOOL GetAccessibleContextInfo(long vmID, AccessibleContext ac, AccessibleContextInfo *info);
AccessibleContext GetAccessibleChildFromContext(long vmID, AccessibleContext ac, jint index);
AccessibleContext GetAccessibleParentFromContext(long vmID, AccessibleContext ac);

/**
 * Accessible Text routines
 */
BOOL GetAccessibleTextInfo(long vmID, AccessibleText at, AccessibleTextInfo *textInfo, jint x, jint y);
BOOL GetAccessibleTextItems(long vmID, AccessibleText at, AccessibleTextItemsInfo *textItems, jint index);
BOOL GetAccessibleTextSelectionInfo(long vmID, AccessibleText at, AccessibleTextSelectionInfo *textSelection);
BOOL GetAccessibleTextAttributes(long vmID, AccessibleText at, jint index, AccessibleTextAttributesInfo *attributes);
BOOL GetAccessibleTextRect(long vmID, AccessibleText at, AccessibleTextRectInfo *rectInfo, jint index);
BOOL GetAccessibleTextLineBounds(long vmID, AccessibleText at, jint index, jint *startIndex, jint *endIndex);
BOOL GetAccessibleTextRange(long vmID, AccessibleText at, jint start, jint end, wchar_t *text, short len);

/* begin AccessibleTable routines */
BOOL getAccessibleTableInfo(long vmID, AccessibleContext acParent, AccessibleTableInfo *tableInfo);

BOOL getAccessibleTableCellInfo(long vmID, AccessibleTable accessibleTable, jint row, jint column, 
								AccessibleTableCellInfo *tableCellInfo);

BOOL getAccessibleTableRowHeader(long vmID, AccessibleContext acParent, AccessibleTableInfo *tableInfo);
BOOL getAccessibleTableColumnHeader(long vmID, AccessibleContext acParent, AccessibleTableInfo *tableInfo);

AccessibleContext getAccessibleTableRowDescription(long vmID, AccessibleContext acParent, jint row);
AccessibleContext getAccessibleTableColumnDescription(long vmID, AccessibleContext acParent, jint column);

jint getAccessibleTableRowSelectionCount(long vmID, AccessibleTable table);
BOOL isAccessibleTableRowSelected(long vmID, AccessibleTable table, jint row);
BOOL getAccessibleTableRowSelections(long vmID, AccessibleTable table, jint count, jint *selections);

jint getAccessibleTableColumnSelectionCount(long vmID, AccessibleTable table);
BOOL isAccessibleTableColumnSelected(long vmID, AccessibleTable table, jint column);
BOOL getAccessibleTableColumnSelections(long vmID, AccessibleTable table, jint count, jint *selections);

jint getAccessibleTableRow(long vmID, AccessibleTable table, jint index);
jint getAccessibleTableColumn(long vmID, AccessibleTable table, jint index);
jint getAccessibleTableIndex(long vmID, AccessibleTable table, jint row, jint column);
/* end AccessibleTable */

BOOL getAccessibleRelationSet(long vmID, AccessibleContext accessibleContext, 
	AccessibleRelationSetInfo *relationSetInfo);

/**
/**
 * Accessible Value routines
 */
BOOL GetCurrentAccessibleValueFromContext(long vmID, AccessibleValue av, wchar_t *value, short len);
BOOL GetMaximumAccessibleValueFromContext(long vmID, AccessibleValue av, wchar_t *value, short len);
BOOL GetMinimumAccessibleValueFromContext(long vmID, AccessibleValue av, wchar_t *value, short len);

/**
 * Accessible Selection routines
 */
void AddAccessibleSelectionFromContext(long vmID, AccessibleSelection as, int i);
void ClearAccessibleSelectionFromContext(long vmID, AccessibleSelection as);
jobject GetAccessibleSelectionFromContext(long vmID, AccessibleSelection as, int i);
int GetAccessibleSelectionCountFromContext(long vmID, AccessibleSelection as);
BOOL IsAccessibleChildSelectedFromContext(long vmID, AccessibleSelection as, int i);
void RemoveAccessibleSelectionFromContext(long vmID, AccessibleSelection as, int i);
void SelectAllAccessibleSelectionFromContext(long vmID, AccessibleSelection as);

#ifdef __cplusplus
}
#endif
