/*
 * Copyright 2002 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * @(#)AccessBridgeInstaller.cpp	1.17 02/08/23
 */

/* 
 * Installer for the Java Access Bridge
 */

#ifdef _DEBUG
	#define DEBUGGING_ON
#endif

#ifndef _MT
    #define _MT
#endif

#include "AccessBridgeInstaller.h"
#include "AccessBridgeInstallerResource.h"
#include "AccessBridgeTester.h"

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <direct.h>
#include <process.h>

// ---------------------------------------------------------------------------

extern "C" {

	HWND theInstallerWindow;
	HINSTANCE theInstance;
   	AccessBridgeInstaller *theInstaller;

    HWND theInstallerSearchingScreen;
    HWND theInstallerVerifyingScreen;

	char theInstallerClassName[] = "InstallerWin";

    HANDLE theThreadRunMutex;
    BOOL theDriveSearchList[26];
}


// ---------------------------------------------------------------------------

extern "C" {

	/**
	 * WinMain
	 *
	 */
	int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, 
					   LPSTR lpCmdLine, int nShowCmd) {
        MSG msg;
        HWND hwnd;

		theInstance = hInst;
		theInstaller = new AccessBridgeInstaller(hInst);
		hwnd = theInstaller->createGUI(nShowCmd);
        PostMessage(hwnd, INSTALLER_START_INSTALLATION, (WPARAM) 0, (LPARAM) 0);

        while (GetMessage(&msg, NULL, 0, 0)) {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }

		return msg.wParam;
	}

// -----------------------------

    void findNextJVM(HWND messageWindow) {
        static struct _finddata_t vm_file;
        static long file;
        static struct _finddata_t directory_file;
        static char currentDirBuf[_MAX_PATH];      // recursion OK, re-entrancy is NOT!
        static char statusString[_MAX_PATH + 50];  // recursion OK, re-entrancy is NOT!
        static LRESULT result;
        long directory;

        if (WaitForSingleObject(theThreadRunMutex, 0L) != WAIT_TIMEOUT) {
            return;
        }

        /* Find 'java.exe" file in current directory */
        if ((file = _findfirst("java.exe", &vm_file )) != -1L) {
            // we found one!!!
            _getcwd(currentDirBuf, _MAX_PATH);
            result = SendMessage(messageWindow,
                        INSTALLER_FOUND_DIR_MESSAGE, 
                        (WPARAM) 0, (LPARAM) currentDirBuf);      // let the installer know
        } else {
            // none in this directory; recurse down:
            _findclose(file);   // close our find for "java.exe"

            if ((directory = _findfirst("*.*", &directory_file)) != -1L) {
                if (directory_file.attrib &  _A_SUBDIR && directory_file.name[0] != '.') {
                    _getcwd(currentDirBuf, _MAX_PATH);
                    result = SendMessage(messageWindow,
                                INSTALLER_SEARCHING_DIR_MESSAGE, 
                                (WPARAM) 0, (LPARAM) currentDirBuf);
                    _chdir(directory_file.name);
                    findNextJVM(messageWindow);
                    _chdir("..");
                }
                while ( _findnext( directory, &directory_file ) == 0) {
                    if (directory_file.attrib &  _A_SUBDIR && directory_file.name[0] != '.') {
                        _getcwd(currentDirBuf, _MAX_PATH);
                        result = SendMessage(messageWindow,
                                    INSTALLER_SEARCHING_DIR_MESSAGE, 
                                    (WPARAM) 0, (LPARAM) currentDirBuf);
                        _chdir(directory_file.name);
                        findNextJVM(messageWindow);
                        _chdir("..");
                    }
                }
            }
            _findclose(directory);
        }
    }

// -----------------------------

    void findJVMsThread(void *ptr) {
        // walk through drive letters in some intelligent fashion
        // foreach letter, cd to root & call findNextJVM() on it

        HWND messageWindow = theInstallerSearchingScreen;

        int initialDrive;
        char initialPath[_MAX_PATH];
        initialDrive = _getdrive();     // to restore it later
        _getcwd(initialPath, _MAX_PATH);

        int drive;

        for (drive = 1; drive <= 26; drive++) {
            if (theDriveSearchList[drive-1] == TRUE) {
                if (!_chdrive(drive)) {
                    // start in root
                    _chdir("\\");
                    findNextJVM(messageWindow);
                }
            }
        }

        _chdrive(initialDrive);  // redundant with _chdir() below?
        _chdir(initialPath);

        PostMessage(messageWindow,
                    INSTALLER_SEARCH_FINISHED, 
                    (WPARAM) 0, (LPARAM) 0);
    }

// -----------------------------

    BOOL CALLBACK InstallerSearchingDialogProc(HWND hWnd, UINT message, 
                                               UINT wParam, LONG lParam) {
        JVM *foundJVM;
	    int command;

	    switch (message) {
        case WM_INITDIALOG:
            theInstallerSearchingScreen = hWnd;     // parameter passing in _beginthread() not working
            theThreadRunMutex = CreateMutex( NULL, TRUE, NULL );    // starts set
            _beginthread(findJVMsThread, 0, (void *) &hWnd);
            break;

	    case WM_CLOSE:
            EndDialog(hWnd, cCancelResult);
		    break;
	    	    
	    case WM_COMMAND:
		    command = LOWORD(wParam);
		    switch(command) {
            case cCancelID:
                CloseHandle(theThreadRunMutex);     // time to end things...
                EndDialog(hWnd, cCancelResult);
                return 1;
            }
            break;


        case INSTALLER_SEARCHING_DIR_MESSAGE:
            SetDlgItemText(hWnd, cCurrentDirTextID, (char *) lParam);
            return 1;

        case INSTALLER_FOUND_DIR_MESSAGE:
            foundJVM = new JVM(theInstaller->JVMs, (char *) lParam);
            if (foundJVM != (JVM *) 0) {
                theInstaller->JVMs = foundJVM;
            }
            return 1;

        case INSTALLER_SEARCH_FINISHED:
            CloseHandle(theThreadRunMutex);     // time to end things...
            EndDialog(hWnd, cInstallResult);
            return 1;
        }

	    return 0;   
    }

// -----------------------------

    BOOL CALLBACK InstallerSelectDisksDialogProc(HWND hWnd, UINT message, 
                                                 UINT wParam, LONG lParam) {
	    int command;
        static int drives[26];
        int i;
        int driveCount;
        int selections[26];
        int selCount;
        char s[256];

	    switch (message) {
        case WM_INITDIALOG:
            // populate the list with drive strings, only of
            // those drives available on the system

            int initialDrive;
            char initialPath[_MAX_PATH];
            initialDrive = _getdrive();     // to restore it later
            _getcwd(initialPath, _MAX_PATH);

            driveCount = 0;
            for (i = 3; i <= 26; i++) {     // no JVMs on floppies; drive 'A' == 1
                if (!_chdrive(i)) {
                    drives[driveCount++] = i;
               }
            }

            _chdrive(initialDrive);  // redundant with _chdir() below?
            _chdir(initialPath);

            for (i = 0; i < driveCount; i++) {
                sprintf(s, "[%c:]", (char) ('A' + drives[i] - 1));
                SendDlgItemMessage(hWnd, cDrivesListID, LB_ADDSTRING, 
                                   (WPARAM) 0, (LPARAM) s);
            }
            break;

	    case WM_CLOSE:
            EndDialog(hWnd, cCancelResult);
		    break;
	    	    
	    case WM_COMMAND:
		    command = LOWORD(wParam);
		    switch(command) {
            case cCancelID:
                EndDialog(hWnd, cCancelResult);
                return 1;
            case cSearchID:
                // get list of drives, store it somewhere...
                // close dialog and invoke the searching function...
                for (i = 0; i < 26; i++ ) {
                    theDriveSearchList[i] = FALSE;
                }
                selCount = SendDlgItemMessage(hWnd, cDrivesListID,
                                              LB_GETSELITEMS, 
                                              (WPARAM) 26, (LPARAM) selections);
                for (i = 0; i < selCount; i++) {
                    theDriveSearchList[drives[selections[i]]-1] = TRUE;
                }
                EndDialog(hWnd, cSearchID);
                return 1;
            }
            break;
        }

	    return 0;   
    }

// -----------------------------

    void verifyJVMsThread(void *ptr) {
        // walk through JVMs list, call JVM->testAccessBridgeCompatibility() on each
        int result;
        HWND messageWindow = theInstallerVerifyingScreen;

        JVM *current = theInstaller->JVMs;
        while (current != (JVM *) 0) {
            if (WaitForSingleObject(theThreadRunMutex, 0L) != WAIT_TIMEOUT) {
                return;
            }
            if (current->usable == FALSE) {             // we may have tested it already...
                result = SendMessage(messageWindow,
                            INSTALLER_SEARCHING_DIR_MESSAGE, 
                            (WPARAM) 0, (LPARAM) current->path);
                current->testAccessBridgeCompatibility();
            }
            current = current->next;
        }
        result = PostMessage(messageWindow,
                    INSTALLER_SEARCH_FINISHED, 
                    (WPARAM) 0, (LPARAM) 0);
        return;
    }

// -----------------------------

    BOOL CALLBACK InstallerVerifyingDialogProc(HWND hWnd, UINT message, 
                                               UINT wParam, LONG lParam) {
        JVM *foundJVM;
	    int command;

	    switch (message) {
        case WM_INITDIALOG:
            theInstallerVerifyingScreen = hWnd;     // parameter passing in _beginthread() not working
            theThreadRunMutex = CreateMutex( NULL, TRUE, NULL );    // starts set
            _beginthread(verifyJVMsThread, 0, (void *) &hWnd);
            break;

	    case WM_CLOSE:
            EndDialog(hWnd, cCancelResult);
		    break;
	    	    
	    case WM_COMMAND:
		    command = LOWORD(wParam);
		    switch(command) {
            case cCancelID:
                CloseHandle(theThreadRunMutex);     // time to end things...
                EndDialog(hWnd, cCancelResult);
                return 1;
            }
            break;

        case INSTALLER_SEARCHING_DIR_MESSAGE:
            SetDlgItemText(hWnd, cCurrentDirTextID, (char *) lParam);
            return 1;

        case INSTALLER_FOUND_DIR_MESSAGE:
            foundJVM = new JVM(theInstaller->JVMs, (char *) lParam);
            if (foundJVM != (JVM *) 0) {
                theInstaller->JVMs = foundJVM;
            }
            return 1;

        case INSTALLER_SEARCH_FINISHED:
            CloseHandle(theThreadRunMutex);     // time to end things...
            EndDialog(hWnd, cInstallResult);
            return 1;
        }

	    return 0;   
    }


// -----------------------------

    BOOL CALLBACK InstallerNoVMsSearchDialogProc(HWND hWnd, UINT message, 
                                                 UINT wParam, LONG lParam) {
	    int command;

	    switch (message) {
        case WM_INITDIALOG:
            break;

	    case WM_CLOSE:
            EndDialog(hWnd, cCancelResult);
		    break;
	    	    
	    case WM_COMMAND:
		    command = LOWORD(wParam);
		    switch(command) {
            case cCancelID:
                EndDialog(hWnd, cCancelResult);
                return 1;
            case cSearchID:
                EndDialog(hWnd, cSearchMoreResult);
                return 1;
            }
            break;
        }

	    return 0;   
    }


// -----------------------------

    BOOL CALLBACK InstallerJVMPathDialogProc(HWND hWnd, UINT message, 
                                             UINT wParam, LONG lParam) {
        static JVM **JVMarray;
        static int jvmArraySize;
        JVM *current;
	    int command;
        int count;
        int i;
        int *itemnumbers;

	    switch (message) {
        case WM_INITDIALOG:

            // make an array out of the current JVM list...
            jvmArraySize = 0;
            current = theInstaller->JVMs;
            while (current != (JVM *) 0) {
                jvmArraySize++;
                current = current->next;
            }
            JVMarray = (JVM **) malloc(jvmArraySize * sizeof(JVM *));     // allocate array of pointers
            current = theInstaller->JVMs;
            for (i = 0; i < jvmArraySize; i++ ) {
                JVMarray[i] = current;
                current = current->next;
            }

            // populate the list with JVM strings,
            // [graying out those that don't support the AccessBridge??]
            for (i = 0; i < jvmArraySize; i++) {
                SendDlgItemMessage(hWnd, cJVMListID, LB_ADDSTRING, 
                                   (WPARAM) 0, (LPARAM) JVMarray[i]->path);
            }
            break;

	    case WM_CLOSE:
            free(JVMarray);
            EndDialog(hWnd, cCancelResult);
		    break;
	    	    
	    case WM_COMMAND:
		    command = LOWORD(wParam);
		    switch(command) {
            case cCancelID:
                free(JVMarray);
                EndDialog(hWnd, cCancelResult);
                return 1;

            case cJVMListID:
                // once the user selects something, enable Install button
                EnableWindow(GetDlgItem(hWnd, cInstallID), TRUE);
                return 0;

            case cInstallID:
                // mark JVMs as installInto = TRUE if they are selected
                // when the user hits the "Install" button...

                count = SendDlgItemMessage(hWnd, cJVMListID, LB_GETSELCOUNT, 
                                               (WPARAM) 0, (LPARAM) 0);
                itemnumbers = (int *) malloc(count * sizeof(int *));
                SendDlgItemMessage(hWnd, cJVMListID, LB_GETSELITEMS, 
                                   (WPARAM) count, (LPARAM) itemnumbers);
                
                for (i = 0; i < count; i++) {
                    JVMarray[itemnumbers[i]]->installInto = TRUE;
                }

                free(itemnumbers);
                free(JVMarray);
                EndDialog(hWnd, cInstallResult);
                return 1;

            case cInstallInAllID:
                // mark all JVMs as installInto = TRUE
                // when the user hits the "Install in All" button...

                for (i = 0; i < jvmArraySize; i++ ) {
                    JVMarray[i]->installInto = TRUE;
                }

                free(JVMarray);
                EndDialog(hWnd, cInstallResult);
                return 1;

            case cSearchID:
                free(JVMarray);
                EndDialog(hWnd, cSearchMoreResult);
                return 1;
           }
            break;
        }

	    return 0;   
    }

// -----------------------------

    LRESULT CALLBACK InstallerWindowProc(HWND hWnd, UINT message, 
                                         UINT wParam, LONG lParam) {
	    switch (message) {

        case WM_DESTROY:
            PostQuitMessage(0);
            break;

        case INSTALLER_START_INSTALLATION:
            theInstaller->install();
            PostQuitMessage(0);
            break;

        default:
            return DefWindowProc(hWnd, message, wParam, lParam);
        }
        return 0;
    }

}

// ---------------------------------------------------------------------------

/**
 * Get the thing going
 *
 */
AccessBridgeInstaller::AccessBridgeInstaller(HINSTANCE hInstance) {
    JVMs = (JVM *) 0;
    paths = (PathList *) 0;
    _getcwd(launchDirectory, _MAX_DIR);
}


// -----------------------------

/**
 * Do the actual installation work
 *
 */
void
AccessBridgeInstaller::install() {
    int result;

    unsetCLASSPATH();

    // check the environment PATH, Registry, etc. first
    if ((result = findJVMsInStandardPlaces()) != cCancelResult) {
        
        pruneJVMs();    // unique the list

        // did we get any JVMs?  different dialogs depending on the answer
        if (JVMs == (JVM *) 0) {
            // no JVMs found that work; ask user what to do!
            result = askUserToSearchDrive();
        } else {
            // does the user want to cancel, search entire disk, or install into stuff on path?
            result = getPathInstallListFromUser();
        }

        while (result != cCancelResult) {
            switch (result) {
            case cSearchMoreResult:
                if ((result = findJVMsOnDisk()) == cCancelResult) {
                    break;
                }
                pruneJVMs();    // unique the list

                // did we get any JVMs?  perhaps the user wants to look more thoroughly...
                if (JVMs == (JVM *) 0) {
                    result = askUserToSearchDrive();
                    break;  // loop back to either cSearchMoreResult, cInstallResult, or cCancelResult
                } else {
                    if ((result = getDiskInstallListFromUser()) == cCancelResult) {
                        break;
                    }
                }

                // did we get any JVMs?  perhaps the user wants to look more thoroughly...
                if (JVMs == (JVM *) 0) {
                    result = askUserToSearchDrive();
                    break;  // loop back to either cSearchMoreResult, cInstallResult, or cCancelResult
                }

                // intentional fall-through to doInstall() below

            case cInstallResult:
                result = doInstall();
                if (result == 1) {
                    showDone(TRUE, TRUE);
                } else if (result == 0) {
                    showDone(TRUE, FALSE);
                } else {
                    showDone(FALSE, FALSE);
                }
                return;
            }
        }
    }

    // show done screen
    showDone(FALSE, FALSE);
}



// -----------------------------

/**
 * Make sure the CLASSPATH environment variable is unset for the installer
 *
 * We do it explicitly this way, rather than using _spawnle(), because
 * we want the rest of the environment to remain the same.
 *
 */
void
AccessBridgeInstaller::unsetCLASSPATH() {
    char *envvar = getenv("CLASSPATH");

    if (envvar != NULL) {
       _putenv("CLASSPATH=");
    }
}

// -----------------------------

/**
 * Build a list of JVMs
 *
 */
HWND
AccessBridgeInstaller::createGUI(int showCommand) {
    WNDCLASSEX wcl;
    HWND hwnd;
    char windowName[] = "Installer Window";

    wcl.cbSize = sizeof(WNDCLASSEX);
    wcl.hInstance = theInstance;
    wcl.lpszClassName = windowName;
    wcl.lpfnWndProc = InstallerWindowProc;
    wcl.style = 0;
    wcl.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wcl.hIconSm = LoadIcon(NULL, IDI_WINLOGO);
    wcl.hCursor = LoadCursor(NULL, IDC_ARROW);
    wcl.lpszMenuName = 0;   // no menu
    wcl.cbClsExtra = 0;
    wcl.cbWndExtra = 0;

    wcl.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);

    if(!RegisterClassEx(&wcl)) {
        return NULL;
    } 
    
    hwnd = CreateWindow(windowName,
                          "Java Access Bridge for Microsoft Windows installer",
                          WS_OVERLAPPEDWINDOW,
                          CW_USEDEFAULT,   /* X coordinate - let Windows decide */
                          CW_USEDEFAULT,   /* Y coordinate - let Windows decide */
                          CW_USEDEFAULT,   /* width coordinate - let Windows decide */
                          CW_USEDEFAULT,   /* height coordinate - let Windows decide */
                          HWND_DESKTOP,
                          NULL,
                          theInstance,
                          NULL);
//    ShowWindow(hwnd, showCommand);
//    UpdateWindow(hwnd);
    return hwnd;
}


// -----------------------------

/**
 * Build a list of JVMs
 *
 */
int
AccessBridgeInstaller::findJVMsInStandardPlaces() {

  // build list of JVMs on path

    buildEnvironmentPathList();
    addRegistryPathList();
    scanEnvironmentPathList();

  // verify AccessBridge compatiblity

    int result = DialogBox(theInstance,
                           "INSTALLERVERIFYINGJVMSCREEN", NULL,
			               (DLGPROC) InstallerVerifyingDialogProc);
 
    return result;
}


// -----------------------------

/**
 * Build a list of JVMs
 *
 */
int
AccessBridgeInstaller::findJVMsOnDisk() {

  // get list of Drives to search on
  // result stored in global (yeech!) theDriveSearchList

    int result = DialogBox(theInstance,
                           "INSTALLERDISKTOSEARCHIN", NULL,
			               (DLGPROC) InstallerSelectDisksDialogProc);
 
    if (result == cCancelResult) {
        // the user let the search complete
        return result;
    }

  // build list of JVMs on drvies in theDriveSearchList

    result = DialogBox(theInstance,
                           "INSTALLERSEARCHINGSCREEN", NULL,
			               (DLGPROC) InstallerSearchingDialogProc);
 
    if (result == cCancelResult) {
        // the user let the search complete
        return result;
    }

  // verify AccessBridge compatiblity

    result = DialogBox(theInstance,
                       "INSTALLERVERIFYINGJVMSCREEN", NULL,
			           (DLGPROC) InstallerVerifyingDialogProc);
 
    return result;
}


// -----------------------------

/**
 * Build the list of paths in the environment variable 'PATH'
 *
 */
BOOL
AccessBridgeInstaller::buildEnvironmentPathList() {
    char *path;
    path = getenv("PATH");
    if (path == NULL) {
        return FALSE;
    }
    
    // is this necessary?  seemed to be in some cases...
    char dummy[2];
    DWORD size = ExpandEnvironmentStrings(path, dummy, 2);
    char *expandedPath = (char *) malloc(size + 1);
    ExpandEnvironmentStrings(path, expandedPath, size);

  // parse path into elements
    PathList *current = (PathList *) 0;
    char *token = strtok(expandedPath, ";");
    while (token != NULL) {
        current = new PathList(current, token);
        token = strtok(NULL, ";");
    }
    paths = current;

    free(expandedPath);     // Oh my Toto, we're not in Java anymore...

    return TRUE;
}


// -----------------------------

#define ADD_PATH_KEY(regPath)                                                       \
    result = RegOpenKeyEx(HKEY_LOCAL_MACHINE,                                       \
                          regPath,                                                  \
                          0,                                                        \
                          KEY_READ,                                                 \
                          &key);                                                    \
    if (result == ERROR_SUCCESS) {                                                  \
        result = RegQueryValueEx(key, "JavaHome", 0, NULL, NULL, &maxKeyValueLen);  \
        javaHomeLen = maxKeyValueLen + strlen("\\bin") + 1;                             \
        javaHomeStr = (char *) malloc(javaHomeLen + 128);                                 \
        result = RegQueryValueEx(key, "JavaHome", 0, NULL,                          \
                                 (unsigned char *) javaHomeStr, &maxKeyValueLen);   \
        strncat(javaHomeStr, "\\bin", javaHomeLen);                                 \
        current = new PathList(paths, javaHomeStr);                                 \
        paths = current;                                                            \
        free(javaHomeStr);                                                          \
        result = RegCloseKey(key);                                                  \
    }


/**
 * Add to the PathList those directories containing JVMs
 * as indicated in the Registry
 *
 */
BOOL
AccessBridgeInstaller::addRegistryPathList() {
    HKEY key;
    long result;
    DWORD maxKeyValueLen;
    char *javaHomeStr;
    int javaHomeLen;
    PathList *current;

    ADD_PATH_KEY("Software\\JavaSoft\\Java Development Kit\\1.2");
    ADD_PATH_KEY("Software\\JavaSoft\\Java Development Kit\\1.3");
    ADD_PATH_KEY("Software\\JavaSoft\\Java Development Kit\\1.3.0");
    ADD_PATH_KEY("Software\\JavaSoft\\Java Development Kit\\1.3.0_01");
    ADD_PATH_KEY("Software\\JavaSoft\\Java Development Kit\\1.3.1");
    ADD_PATH_KEY("Software\\JavaSoft\\Java Development Kit\\1.3.1_01");
    ADD_PATH_KEY("Software\\JavaSoft\\Java Development Kit\\1.4");
    ADD_PATH_KEY("Software\\JavaSoft\\Java Development Kit\\1.4.0");
	ADD_PATH_KEY("Software\\JavaSoft\\Java Development Kit\\1.4.0_01");
	ADD_PATH_KEY("Software\\JavaSoft\\Java Development Kit\\1.4.1");
	ADD_PATH_KEY("Software\\JavaSoft\\Java Development Kit\\1.4.1_02");
    ADD_PATH_KEY("Software\\JavaSoft\\Java Development Kit\\1.4.2");

    ADD_PATH_KEY("Software\\JavaSoft\\Java Runtime Environment\\1.2");
    ADD_PATH_KEY("Software\\JavaSoft\\Java Runtime Environment\\1.3");
    ADD_PATH_KEY("Software\\JavaSoft\\Java Runtime Environment\\1.3.0");
    ADD_PATH_KEY("Software\\JavaSoft\\Java Runtime Environment\\1.3.0_01");
    ADD_PATH_KEY("Software\\JavaSoft\\Java Runtime Environment\\1.3.1");
    ADD_PATH_KEY("Software\\JavaSoft\\Java Runtime Environment\\1.3.1_01");
    ADD_PATH_KEY("Software\\JavaSoft\\Java Runtime Environment\\1.4");
    ADD_PATH_KEY("Software\\JavaSoft\\Java Runtime Environment\\1.4.0");
	ADD_PATH_KEY("Software\\JavaSoft\\Java Runtime Environment\\1.4.0_01");
	ADD_PATH_KEY("Software\\JavaSoft\\Java Runtime Environment\\1.4.1");
	ADD_PATH_KEY("Software\\JavaSoft\\Java Runtime Environment\\1.4.1_02");
    ADD_PATH_KEY("Software\\JavaSoft\\Java Runtime Environment\\1.4.2");

    return TRUE;
}

// -----------------------------

/**
 * Scan PathList *paths for JVMs
 *
 */
BOOL
AccessBridgeInstaller::scanEnvironmentPathList() {
    PathList *current = paths;
    char *aPath;
    while (current != NULL) {
        aPath = current->getPath();
        if (isJVMContainingDirectory(aPath)) {
            JVMs = new JVM(JVMs, aPath);
        }
        current = current->getNext();
    } 

    return TRUE;
}


// -----------------------------

/**
 * Build a list of JVMs
 *
 */
BOOL
AccessBridgeInstaller::isJVMContainingDirectory(char *path) {
    struct _finddata_t vm_file;
    long file;
    static char currentDirBuf[_MAX_PATH];      // recursion OK, re-entrancy is NOT!

    if (path == (char *) 0 || !isalpha(path[0])) {
        return FALSE;
    }

    /* Find 'java.exe" file in current directory */
    _getcwd(currentDirBuf, _MAX_PATH);
    _chdir(path);

    file = _findfirst("java.exe", &vm_file);

    if (file != -1L) {
        // we found one!!!
        _chdir(currentDirBuf);
        return TRUE;
    } else {
        // gotta check for "jre.exe" for 1.1 JRE case...
        file = _findfirst("jre.exe", &vm_file);
        _chdir(currentDirBuf);
        if (file != -1L) {
            return TRUE;
        } else {
            return FALSE;
        }
    }
}



// -----------------------------

/**
 * Winnow down the list of JVMs
 *
 * Note: some JVMs have already gone through this process once
 * in the initial screening
 *
 */
BOOL
AccessBridgeInstaller::pruneJVMs() {

  // step 1: build partial paths for stuff

    JVM *current = JVMs;
    while (current != (JVM *) 0) {
        current->buildPartialPaths();
        current = current->next;
    }

  // step 2: remove JVMs which we can't install into

    // first get first one set properly...
    JVM *temp;
    while (JVMs != (JVM *) 0 && (JVMs->usable == FALSE || JVMs->installable == FALSE)) {
        temp = JVMs;
        JVMs = JVMs->next;
        delete temp;            // Oh my Toto, we're not in Java anymore...
    }
    
    // take care of the rest
    current = JVMs;
    if (current != (JVM *) 0) {
        while (current->next != (JVM *) 0) {
            if (current->next->usable == FALSE || current->next->installable == FALSE) {
                temp = current->next;
                current->next = current->next->next;
                delete temp;    // Oh my Toto, we're not in Java anymore...
            } else {
                current = current->next;
            }
        }
    }

  // step 3: unique the remaining JVMs
    // binary unique algorythm OK 'cause we don't expect to have many of these...
    if (JVMs != (JVM *) 0 && JVMs->next != (JVM *) 0) {
        // need to have at least two things in order to unique 'em!
        JVM *compareTo = JVMs;
        current = compareTo;
        while (current->next != (JVM *) 0) {
            if (_strcmpi(compareTo->path, current->next->path) == 0) {
                // we've got a match, remove the offender!
                temp = current->next;
                current->next = current->next->next;
                delete temp;    // Oh my Toto, we're not in Java anymore...
            } else {
                current = current->next;
            }
        }
    }
    
    return TRUE;
}


// -----------------------------

/**
 * Tell user no JVMs found; ask if they want to search drive
 *
 */
int
AccessBridgeInstaller::askUserToSearchDrive() {
  // ask user to install from path list

    int result = DialogBox(theInstance,
                           "INSTALLERNOVMSSCREEN", NULL,
			               (DLGPROC) InstallerNoVMsSearchDialogProc);
 
    return result;
}


// -----------------------------

/**
 * Build a list of JVMs
 *
 */
int
AccessBridgeInstaller::getPathInstallListFromUser() {
  // ask user to install from path list

    int result = DialogBox(theInstance,
                           "INSTALLERJVMPATHLISTSCREEN", NULL,
			               (DLGPROC) InstallerJVMPathDialogProc);
 
    return result;
}


// -----------------------------

/**
 * Build a list of JVMs
 *
 */
int
AccessBridgeInstaller::getDiskInstallListFromUser() {
    int result = DialogBox(theInstance,
                           "INSTALLERJVMLISTSCREEN", NULL,
			               (DLGPROC) InstallerJVMPathDialogProc);
 
    return result;
}


// -----------------------------

/**
 * Build a list of JVMs
 *
 */
int
AccessBridgeInstaller::doInstall() {
    char src[_MAX_DIR];
    char dest[_MAX_DIR];
    char jarsFor11[_MAX_DIR];

    jarsFor11[0] = '\0';

  // copy jar files, property files, appropriate for each JVM we're installing into
    BOOL did11Install = FALSE;
    BOOL mustReboot = FALSE;
    JVM *current = JVMs;
    while (current != (JVM *) 0) {
        if (current->installInto) {
            // we're gonna install into this JVM!
            if ((!current->usable) || (!current->installable)) {
                return -1;   // quick sanity check!
            }
            if (current->has1_3apis) {
                // jdk 1.3 or later...

                  // copy accessibility.properties into %toppath%%libpath%
                strncpy(src, launchDirectory, _MAX_DIR);
                strncat(src, "\\installerFiles", _MAX_DIR);
                strncat(src, "\\accessibility.properties", _MAX_DIR);
                strncpy(dest, current->toppath, _MAX_DIR);
                strncat(dest, current->libpath, _MAX_DIR);
                strncat(dest, "\\accessibility.properties", _MAX_DIR);
                CopyFile(src, dest, FALSE);

                  // copy jaccess-1_4.jar into %toppath%%jarpath%
                strncpy(src, launchDirectory, _MAX_DIR);
                strncat(src, "\\installerFiles", _MAX_DIR);
                strncat(src, "\\jaccess-1_4.jar", _MAX_DIR);
                strncpy(dest, current->toppath, _MAX_DIR);
                strncat(dest, current->jarpath, _MAX_DIR);
                strncat(dest, "\\jaccess.jar", _MAX_DIR);
                CopyFile(src, dest, FALSE);

                  // copy access-bridge.jar into %toppath%%jarpath%
                strncpy(src, launchDirectory, _MAX_DIR);
                strncat(src, "\\installerFiles", _MAX_DIR);
                strncat(src, "\\access-bridge.jar", _MAX_DIR);
                strncpy(dest, current->toppath, _MAX_DIR);
                strncat(dest, current->jarpath, _MAX_DIR);
                strncat(dest, "\\access-bridge.jar", _MAX_DIR);
                CopyFile(src, dest, FALSE);

            } else if (current->needs_EQM) {
                // jdk 1.1.x

                  // edit awt.properties
                strncpy(src, current->toppath, _MAX_DIR);
                strncat(src, current->jarpath, _MAX_DIR);
                modifyAWTproperties(src, "\\awt.properties");

                  // if first 1.1 JVM we're installing into, mark it for jars and CLASSPATH
                if (jarsFor11[0] == '\0') {
                    strncpy(jarsFor11, current->toppath, _MAX_DIR);
                    strncat(jarsFor11, current->jarpath, _MAX_DIR);
                }

                did11Install = TRUE;    // we'll need to modify autoexec.bat... sigh

            } else {
                // jdk 1.2.x

                  // copy accessibility.properties into %toppath%%libpath%
                strncpy(src, launchDirectory, _MAX_DIR);
                strncat(src, "\\installerFiles", _MAX_DIR);
                strncat(src, "\\accessibility.properties", _MAX_DIR);
                strncpy(dest, current->toppath, _MAX_DIR);
                strncat(dest, current->libpath, _MAX_DIR);
                strncat(dest, "\\accessibility.properties", _MAX_DIR);
                CopyFile(src, dest, FALSE);

                  // copy jaccess-1_2.jar into %toppath%%jarpath%
                strncpy(src, launchDirectory, _MAX_DIR);
                strncat(src, "\\installerFiles", _MAX_DIR);
                strncat(src, "\\jaccess-1_2.jar", _MAX_DIR);
                strncpy(dest, current->toppath, _MAX_DIR);
                strncat(dest, current->jarpath, _MAX_DIR);
                strncat(dest, "\\jaccess.jar", _MAX_DIR);
                CopyFile(src, dest, FALSE);

                  // copy access-bridge.jar into %toppath%%jarpath%
                strncpy(src, launchDirectory, _MAX_DIR);
                strncat(src, "\\installerFiles", _MAX_DIR);
                strncat(src, "\\access-bridge.jar", _MAX_DIR);
                strncpy(dest, current->toppath, _MAX_DIR);
                strncat(dest, current->jarpath, _MAX_DIR);
                strncat(dest, "\\access-bridge.jar", _MAX_DIR);
                CopyFile(src, dest, FALSE);
            }
        }
        current = current->next;
    }

  // copy DLLs into windows dir
    char sysDir[_MAX_PATH];

      // copy JavaAccessBridge.dll into \Windows\System directory
    GetSystemDirectory(sysDir, _MAX_PATH);
    strncpy(src, launchDirectory, _MAX_DIR);
    strncat(src, "\\installerFiles", _MAX_DIR);
    strncat(src, "\\JavaAccessBridge.dll", _MAX_DIR);
    strncpy(dest, sysDir, _MAX_DIR);
    strncat(dest, "\\JavaAccessBridge.dll", _MAX_DIR);
    CopyFile(src, dest, FALSE);

      // copy WindowsAccessBridge.dll into \Windows\System directory
    GetSystemDirectory(sysDir, _MAX_PATH);
    strncpy(src, launchDirectory, _MAX_DIR);
    strncat(src, "\\installerFiles", _MAX_DIR);
    strncat(src, "\\WindowsAccessBridge.dll", _MAX_DIR);
    strncpy(dest, sysDir, _MAX_DIR);
    strncat(dest, "\\WindowsAccessBridge.dll", _MAX_DIR);
    CopyFile(src, dest, FALSE);

	  // copy JAWTAccessBridge.dll into \Windows\System directory
    GetSystemDirectory(sysDir, _MAX_PATH);
    strncpy(src, launchDirectory, _MAX_DIR);
    strncat(src, "\\installerFiles", _MAX_DIR);
    strncat(src, "\\JAWTAccessBridge.dll", _MAX_DIR);
    strncpy(dest, sysDir, _MAX_DIR);
    strncat(dest, "\\JAWTAccessBridge.dll", _MAX_DIR);
    CopyFile(src, dest, FALSE);

  // modify autoexec.bat if necessary...
    // (use jarsFor11 variable for CLASSPATH)
    if (did11Install) {
        // *** note: if no swing.jar, then jaccess utilities will have a problem!!! ***
        // ***       similarly, if no ".;classes.zip", will have a problem!!!       ***

        // if we're running in NT, modify registry
        OSVERSIONINFO version;
        version.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
        GetVersionEx(&version);
        if (version.dwPlatformId & VER_PLATFORM_WIN32_NT) {
            HKEY key;
            DWORD disposition;
            long result;
            DWORD maxKeyValueLen;
            char *classpathStr;
            int classpathLen;
            char temp[_MAX_DIR];

            // get/create the ENVIRONMENT key
            result = RegCreateKeyEx(HKEY_LOCAL_MACHINE,
                           "System\\CurrentControlSet\\Control\\Session Manager\\Environment",
                           0, 
                           "",
                           REG_OPTION_NON_VOLATILE,
                           KEY_ALL_ACCESS,
                           NULL,
                           &key, 
                           &disposition);

            // get size of CLASSPATH data element in ENVIRONMENT key
            result = RegQueryValueEx(key, "CLASSPATH", 0, NULL, NULL, &maxKeyValueLen); 
            
            // [[[FIXME]]] if CLASSPATH empty, include "." & "classes.zip"
            //             if CLASSPATH has our stuff on it, leave it alone!

            // malloc a string of that length, plus our potential addition:
            classpathLen = maxKeyValueLen + sizeof(";\\jaccess.jar;\\access-bridge.jar") + (strlen(jarsFor11) *2) + 2;   // extra slop...
            classpathStr = (char *) malloc(classpathLen + 1);

            // get the current CLASSPATH registry string
            result = RegQueryValueEx(key, "CLASSPATH", 0, NULL, (unsigned char *) classpathStr, &maxKeyValueLen);

            // modify/create the CLASSPATH registry string
            sprintf(temp, "%s\\jaccess.jar;%s\\access-bridge.jar", jarsFor11, jarsFor11);
            if (classpathStr[0] != '\0') {
                strncat(classpathStr, ";", classpathLen);
            }
            strncat(classpathStr, temp, classpathLen);

            // set the CLASSPATH registry string
            result = RegSetValueEx(key, "CLASSPATH", 0, REG_SZ, (const unsigned char *) classpathStr, strlen(classpathStr)+1);
            result = RegFlushKey(key);

            free(classpathStr);     // Oh my Toto, we're not in Java anymore...

        } else {
            // else we're in Win95/98, so modify autoexec.bat
            strncpy(src, sysDir, 3);    // put system drive letter in 'src'
            src[3] = '\0';
            modifyAutoexec(src, "autoexec.bat", jarsFor11);
            mustReboot = TRUE;
        }

          // copy jaccess-1_1.jar into %toppath%%jarpath%
        strncpy(src, launchDirectory, _MAX_DIR);
        strncat(src, "\\installerFiles", _MAX_DIR);
        strncat(src, "\\jaccess-1_1.jar", _MAX_DIR);
        strncpy(dest, jarsFor11, _MAX_DIR);
        strncat(dest, "\\jaccess.jar", _MAX_DIR);
        CopyFile(src, dest, FALSE);

          // copy access-bridge.jar into %toppath%%jarpath%
        strncpy(src, launchDirectory, _MAX_DIR);
        strncat(src, "\\installerFiles", _MAX_DIR);
        strncat(src, "\\access-bridge.jar", _MAX_DIR);
        strncpy(dest, jarsFor11, _MAX_DIR);
        strncat(dest, "\\access-bridge.jar", _MAX_DIR);
        CopyFile(src, dest, FALSE);
    }

    if (mustReboot) {
        return 1;
    } else {
        return 0;
    }
}

// -----------------------------

/**
 * Edit the awt.properties file to load AccessBridge
 *
 */
BOOL
AccessBridgeInstaller::modifyAWTproperties(char *dir, char *filename) {
    FILE *propFile;
    FILE *tempFile;
    char propFilename[_MAX_PATH];
    char tempFilename[_MAX_PATH];
    char line[256];
    BOOL didEQM = FALSE;
    BOOL didAB = FALSE;

    strncpy(propFilename, dir, _MAX_PATH);
    strncat(propFilename, filename, _MAX_PATH);
    if ((propFile = fopen(propFilename, "r" )) == NULL) {
      return FALSE;
    }

    strncpy(tempFilename, dir, _MAX_PATH);
    strncat(tempFilename, "\\___AccessBridgeTempFile___", _MAX_PATH);
    if ((tempFile = fopen(tempFilename, "w+" )) == NULL) {
      return FALSE;
    }


    // does our file already have an "AWT.EventQueueClass" line?
    while (fgets(line, sizeof(line), propFile) != NULL) {
        if (_strnicmp(line, "AWT.EventQueueClass=", strlen("AWT.EventQueueClass")) == 0) {
            // write out a modified line
            fputs("AWT.EventQueueClass=com.sun.java.accessibility.util.EventQueueMonitor\n", tempFile);
            didEQM = TRUE;
        } else if (_strnicmp(line, "AWT.assistive_technologies=", strlen("AWT.assistive_technologies=")) == 0) {
            // write out a modified line
            fputs("AWT.assistive_technologies=com.sun.java.accessibility.AccessBridge\n", tempFile);
            didAB = TRUE;
        } else {
            fputs(line, tempFile);
        }
    }

    if (!didEQM) {
        fputs("AWT.EventQueueClass=com.sun.java.accessibility.util.EventQueueMonitor\n", tempFile);
    }
    if (!didAB) {
        fputs("AWT.assistive_technologies=com.sun.java.accessibility.AccessBridge\n", tempFile);
    }

    fclose(propFile);
    fclose(tempFile);

    remove(propFilename);
    rename(tempFilename, propFilename);

    return TRUE;
}


// -----------------------------

/**
 * Edit the AUTOEXEC.BAT file to set the CLASSPATH
 *
 */
BOOL
AccessBridgeInstaller::modifyAutoexec(char *dir, char *filename, char *libpath) {
    FILE *autoexecFile;
    FILE *tempFile;
    char autoexecFilename[_MAX_PATH];
    char tempFilename[_MAX_PATH];
    char line[256];
    char s[256];
    BOOL didCLASSPATH = FALSE;

    strncpy(tempFilename, dir, _MAX_PATH);
    strncat(tempFilename, "\\___AccessBridgeTempFile___", _MAX_PATH);
    if ((tempFile = fopen(tempFilename, "w+" )) == NULL) {
      return FALSE;
    }

    strncpy(autoexecFilename, dir, _MAX_PATH);
    strncat(autoexecFilename, filename, _MAX_PATH);
    if ((autoexecFile = fopen(autoexecFilename, "r")) != NULL) {

        // [[[FIXME]]] if CLASSPATH empty, include "." & "classes.zip"
        //             if CLASSPATH has our stuff on it, leave it alone!


        // does our file already have an "AWT.EventQueueClass" line?
        while (fgets(line, sizeof(line), autoexecFile) != NULL) {
            if (_strnicmp(line, "set CLASSPATH=", strlen("set CLASSPATH=")) == 0) {
                // write out a modified line
                line[strlen(line)-1] = '\0';    // get rid of '\n' at end of line
                sprintf(s, "%s;%s\\jaccess.jar;%s\\access-bridge.jar\n", line, libpath, libpath);
                fputs(s, tempFile);
                didCLASSPATH = TRUE;
            } else {
                fputs(line, tempFile);
            }
        }
        fclose(autoexecFile);
    }

    if (!didCLASSPATH) {
        sprintf(line, "set CLASSPATH=.;%s\\jaccess.jar;%s\\access-bridge.jar;%s\\classes.zip\n", 
                libpath, libpath, libpath); 
        fputs(line, tempFile);
    }

    fclose(tempFile);

    remove(autoexecFilename);
    rename(tempFilename, autoexecFilename);

    return TRUE;
}

// -----------------------------

/**
 * Show "Done" dialog, with either "good" or "bad" text
 *
 */
void
AccessBridgeInstaller::showDone(BOOL successful, BOOL reboot) {

    if (successful) {
        if (reboot) { 
            MessageBox(theInstallerWindow,
                       "Installation of the Java Access Bridge was successful.", 
                       "Installation Completed.  You must reboot your system.", 
                       MB_OK & MB_SYSTEMMODAL);
        } else {
            MessageBox(theInstallerWindow,
                       "Installation of the Java Access Bridge was successful.", 
                       "Installation Completed", 
                       MB_OK & MB_SYSTEMMODAL);
        }
    } else {
        MessageBox(theInstallerWindow,
                   "Installation of the Java Access Bridge was canceled.", 
                   "Installation Canceled", 
                   MB_OK & MB_SYSTEMMODAL);
    }
}

// --------------------------------------------------------------------------


/**
 * Create a JVM reference object
 *
 */
JVM::JVM(JVM *nextJVM, char *vmPath) {
    next = nextJVM;
    usable = FALSE;
    installable = FALSE;
    needs_EQM = FALSE;
    has1_3apis = FALSE;
    installInto = FALSE;

    path = (char *) malloc(strlen(vmPath) + 1);
    strcpy(path, vmPath);
    toppath = (char *) 0;
    libpath = (char *) 0;
    jarpath = (char *) 0;
}

// -----------------------------

/**
 * Destroy a JVM reference object
 *
 */
JVM::~JVM() {
    if (path != (char *) 0) {
        free(path);
    }
    if (toppath != (char *) 0) {
        free(toppath);
    }
    if (libpath != (char *) 0) {
        free(libpath);
    }
    if (jarpath != (char *) 0) {
        free(jarpath);
    }
}

// -----------------------------

/**
 * Verify that we can install a Java Access Bridge into this JVM
 *
 */
BOOL
JVM::testAccessBridgeCompatibility() {

    // invoke java.exe with AccessBridgeTester.class, look for return val
    char *command;
    command = (char *) malloc(strlen(path) + strlen("\\javaw.exe") + 1);
    usable = FALSE;
    if (command != (char *) 0) {
        strcpy(command, path);
        strcat(command, "\\javaw.exe");         // javaw.exe doesn't bring up a DOS window
        int result = _spawnl(_P_WAIT, command,
                            (const char *) "javaw.exe",
                            (const char *) "AccessBridgeTester", NULL );

        if (result <= 0) {
            // perhaps it's JRE 1.1, in which case the command is 'jrew.exe'; sigh...
            strcpy(command, path);
            strcat(command, "\\jrew.exe");         // jrew.exe doesn't bring up a DOS window
            result = _spawnl(_P_WAIT, command,
                            (const char *) "jrew.exe",
                            (const char *) "-cp . AccessBridgeTester", NULL );
            if (result <= 0) {
                return FALSE;
            }
        }
        if (result & AccessBridgeTester_ACCESS_BRIDGE_COMPATIBILITY) {
            usable = TRUE;
        } else {
            usable = FALSE;
        }
        if (result & AccessBridgeTester_AWT_EVENT_LISTENER_COMPATIBILITY) {
            needs_EQM = FALSE;
        } else {
            needs_EQM = TRUE;
        }
        if (result & AccessBridgeTester_HAS_1_3_APIS) {
            has1_3apis = TRUE;
        } else {
            has1_3apis = FALSE;
        }
    }

    return usable;
}


// -----------------------------

/**
 * Build the various path elements for this JVM
 * (Note: this call depends upon the BOOL flags being set first)
 */
BOOL
JVM::buildPartialPaths() {
    int offset = 0;
    char *remainder;
    char *tokensrc = (char *) malloc(strlen(path) + 1);
    strcpy (tokensrc, path);
    char *token = strtok(tokensrc, "\\");
    char temp[_MAX_PATH];
    int len;
    int result;

    // it may have already gotten it's paths build!
    if (toppath != (char *) 0) {
        return TRUE;
    }

    // get current path so we can restore it later on
    char initialPath[_MAX_PATH];
    _getcwd(initialPath, _MAX_PATH);


	// things we expect and are ready to parse:
	//   <path>\jre\java.exe
	//   <path>\bin\java.exe
    //   <path>\JavaSoft\jre\[1.1*|1.2*|1.3*]\bin\java.exe

    _strlwr(path);

    while (token != NULL) {
        if (_strcmpi(token, "bin") == 0) {
            // we are of the form: c:\jdk1.3\bin
            break;
        }
        // move the main tokenizer forward one token
        offset += strlen(token) + 1;    // for the token  && '\' we just passed over
        token = strtok(NULL, "\\");
    }

    if (token == NULL) {
        return FALSE;       // we couldn't parse things out!
    }

    remainder = (char *) (path + offset-1);

    // set toppath
    toppath = (char *) malloc(offset + 1);
    strncpy(toppath, path, offset-1);
    toppath[offset-1] = '\0';

    if (!needs_EQM) {   // should be a 1.2.x or later JVM; mark where the /lib subdirectory is
        // does %toppath%\lib exist? assumes C:/jdk1.x/jre/lib pattern
        strncpy(temp, toppath, _MAX_PATH);
        strncat(temp, "\\jre\\lib", _MAX_PATH);
        if ((result = _chdir(temp)) == 0) {
            len = strlen("\\jre\\lib") + 1;
            libpath = (char *) malloc(len + 1);
            strncpy(libpath, "\\jre\\lib", len);
            libpath[len-1] = '\0';
        } else {
            // does %toppath%\lib exist? assumes C:/Program Files//JavaSoft/jre/1.x/lib pattern
            strncpy(temp, toppath, _MAX_PATH);
            strncat(temp, "\\lib", _MAX_PATH);
            if ((result = _chdir(temp)) == 0) {
                libpath = (char *) malloc(strlen("\\lib") + 1);
                strcpy(libpath, "\\lib");
            } else {
                _chdir(initialPath);     // oops!  we couldn't find it!
                return FALSE;    
            }
        }

        // does %toppath%\%libpath%\ext exist? (based on /lib subdirectory above...
        strncpy(temp, toppath, _MAX_PATH);
        strncat(temp, libpath, _MAX_PATH);
        strncat(temp, "\\ext", _MAX_PATH);
        if ((result = _chdir(temp)) == 0) {
            len = strlen(libpath) + strlen("\\ext") + 1;
            jarpath = (char *) malloc(len + 1);
            strncpy(jarpath, libpath, len);
            strncat(jarpath, "\\ext", len);
            jarpath[len-1] = '\0';
        } else {
            _chdir(initialPath);        // couldn't find it; could be a 1.1.x JVM
            if (needs_EQM || strlen(libpath) == 4) {
                // yup; it's a 1.1.x JVM; put stuff in "\\lib"
                jarpath = (char *) malloc(strlen("\\lib") + 1);
                strcpy(jarpath, "\\lib");
            }
        }
    } else if (needs_EQM || strlen(libpath) == 4) {
        // yup; it's a 1.1.x JVM; put stuff in "\\lib"
        jarpath = (char *) malloc(strlen("\\lib") + 1);
        strcpy(jarpath, "\\lib");
    }


    installable = TRUE;

    _chdir(initialPath);
    return TRUE;
}


// -----------------------------

/**
 * Mark this JVM as one the user wants to have the Access Bridge installed into
 * (return old setting)
 *
 */
BOOL
JVM::setInstallInto(BOOL setting) {
    BOOL returnVal = installInto;
    installInto = setting;
    return returnVal;
}


// -----------------------------

/**
 * Install the JVM 
 *
 */
int
JVM::installAccessBridgeComponents() {
    // copy the various files into the various places...
    return 0;
}


// --------------------------------------------------------------------------


/**
 * Create a JVM reference object
 *
 */
PathList::PathList(PathList *nextPathList, char *nextPath) {
    next = nextPathList;
    path = (char *) malloc(strlen(nextPath) + 1);
    strcpy(path, nextPath);
}

