/*
 * Copyright 2002 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * @(#)AccessBridgeInstaller.h	1.11 02/08/23
 */

/* 
 * Installer for the Java Access Bridge
 */

/*
 * Installer pieces:
 * 
 *   1. build list of JVMs
 *   2. determine if a JVM is usable
 *   3. modify a JDK 1.1 JVM for AccessBridge
 *   4. modify a JDK 1.2/1.3/... JVM for AccessBridge
 *   5. edit autoexec.bat for CLASSPATH variable for JDK 1.1 JVM
 *   6. <always put DLLs into <win>\<sys> directory
 */

#ifdef _DEBUG
	#define DEBUGGING_ON
#endif

#include <windows.h>
#include <stdio.h>

// Used for messages between the dir searching thread
// and the Dialog UI.
// In the SendMessage call, the third param (WPARAM) is
// null, and the fourth param (LPARAM) is a C string
// containing the directory path being searched
#define INSTALLER_SEARCHING_DIR_MESSAGE (WM_USER+0x0100)

// Used for messages between the dir searching thread
// and the Dialog UI.
// In the SendMessage call, the third param (WPARAM) is
// null, and the fourth param (LPARAM) is a C string
// containing the directory path that a JVM was found in.
#define INSTALLER_FOUND_DIR_MESSAGE (WM_USER+0x0101)

// Used for messages between the dir searching thread
// and the Dialog UI.
// In the SendMessage call, the third and fourth parameters
// (WARAM and LPARAM) are null.
#define INSTALLER_SEARCH_FINISHED (WM_USER+0x0102)

// Used for messages between the dir searching thread
// and the Dialog UI.
// In the SendMessage call, the third and fourth parameters
// (WARAM and LPARAM) are null.
#define INSTALLER_START_INSTALLATION (WM_USER+0x0150)



#define cCancelResult       0
#define cInstallResult      1
#define cSearchMoreResult   2

typedef void (*StatusUpdateFP) (char *status);

extern "C" {
	BOOL WINAPI DllMain(HINSTANCE hinstDll, DWORD fdwReason, 
						LPVOID lpvReserved);
    void findNextJVM(HWND messageWindow);
    void findJVMsThread(void *ptr);
    BOOL APIENTRY InstallerSearchingDialogProc(HWND hWnd, UINT message, 
                                               UINT wParam, LONG lParam);
    BOOL APIENTRY InstallerSelectDisksDialogProc(HWND hWnd, UINT message, 
                                                 UINT wParam, LONG lParam);
    void verifyJVMsThread(void *ptr);
    BOOL CALLBACK InstallerVerifyingDialogProc(HWND hWnd, UINT message, 
                                               UINT wParam, LONG lParam);
    BOOL CALLBACK InstallerNoVMsSearchDialogProc(HWND hWnd, UINT message, 
                                                 UINT wParam, LONG lParam);
    BOOL CALLBACK InstallerJVMPathDialogProc(HWND hWnd, UINT message, 
                                             UINT wParam, LONG lParam);
    LRESULT CALLBACK InstallerWindowProc(HWND hWnd, UINT message, 
                                         UINT wParam, LONG lParam);
}

/**
 * The JVM class.
 */
class JVM {
public:
    JVM *next;
    char *path;         // full path to java.exe
    char *toppath;      // full path to top of JVM/JDK/JRE heirarchy
    char *libpath;      // partial path from top lib dir
    char *jarpath;      // partial path from top to jre dir (if 1.2.x or later)
    BOOL usable;        // will the JVM support the AccessBridge calls?
    BOOL installable;   // can we install the AccessBridge into it?
    BOOL needs_EQM;     // will it need us to do AWT_EventQueueMonitor stuff?
    BOOL has1_3apis;    // does it support AccessibleRelation, etc. of 1.3?
    BOOL installInto;   // does the user want us to install into this JVM

public:
	JVM(JVM *next, char *path);
    ~JVM();

    BOOL testAccessBridgeCompatibility();
    BOOL buildPartialPaths();
    int installAccessBridgeComponents();
    BOOL setInstallInto(BOOL setting);  // return old setting
};

/**
 * The pathlist class.
 */
struct PathList {
private:
    PathList *next;
    char *path;

public:
    PathList(PathList *nextPathList, char *nextPath);
    char * getPath() {return path;}
    PathList * getNext() {return next;}
};

/**
 * The AccessBridgeInstaller class.
 */
class AccessBridgeInstaller {
public:
    JVM *JVMs;
    PathList *paths;
    char launchDirectory[_MAX_DIR];

public:
	AccessBridgeInstaller(HINSTANCE hInstance);
    ~AccessBridgeInstaller();

    void install();     // main install routine
    void unsetCLASSPATH();
    HWND createGUI(int showCommand);
    int findJVMsInStandardPlaces();
    int findJVMsOnDisk();
    BOOL buildEnvironmentPathList();
    BOOL addRegistryPathList();
    BOOL scanEnvironmentPathList();
    BOOL isJVMContainingDirectory(char *path);
    BOOL pruneJVMs();   // get rid of dupes, etc.
    int askUserToSearchDrive();
    int getPathInstallListFromUser();
    int getDiskInstallListFromUser();
    int doInstall();
    BOOL modifyAWTproperties(char *dir, char *filename);
    BOOL modifyAutoexec(char *dir, char *filename, char *libpath);
    void showDone(BOOL successful, BOOL reboot);
};
