/*
 * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
 * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/*
 * @(#)AccessBridgePackages.h	1.27 03/09/02
 */

/* 
 * Header file for packages of paramaters passed between Java Accessibility
 * and native Assistive Technologies
 */

#ifndef __AccessBridgePackages_H__
#define __AccessBridgePackages_H__

#include <jni.h>
//#include <javaString.h>
#include <windows.h>

#ifdef __cplusplus
extern "C" {
#endif

// versioning
//
#define ACCESS_BRIDGE_VERSION  L"1.0.4"
#define JAVA_ACCESS_BRIDGE_VERSION      ACCESS_BRIDGE_VERSION
#define WINDOW_ACCESS_BRIDGE_VERSION    ACCESS_BRIDGE_VERSION


#define MAX_STRING_SIZE   1024
#define SHORT_STRING_SIZE   256

/**
 ******************************************************
 *  Java event types
 ******************************************************
 */

#define cPropertyChangeEvent (jlong) 1						// 1

#define cFocusGainedEvent (jlong) 2							// 2
#define cFocusLostEvent (jlong) 4							// 4

#define cCaretUpdateEvent (jlong) 8							// 8

#define cMouseClickedEvent (jlong) 16						// 10
#define cMouseEnteredEvent (jlong) 32						// 20
#define cMouseExitedEvent (jlong) 64						// 40
#define cMousePressedEvent (jlong) 128						// 80
#define cMouseReleasedEvent (jlong) 256						// 100

#define cMenuCanceledEvent (jlong) 512						// 200
#define cMenuDeselectedEvent (jlong) 1024					// 400
#define cMenuSelectedEvent (jlong) 2048						// 800
#define cPopupMenuCanceledEvent (jlong) 4096				// 1000
#define cPopupMenuWillBecomeInvisibleEvent (jlong) 8192		// 2000
#define cPopupMenuWillBecomeVisibleEvent (jlong) 16384		// 4000


/**
 ******************************************************
 *  Accessibility event types
 ******************************************************
 */

#define cPropertyNameChangeEvent (jlong) 1						// 1
#define cPropertyDescriptionChangeEvent (jlong) 2				// 2
#define cPropertyStateChangeEvent (jlong) 4						// 4
#define cPropertyValueChangeEvent (jlong) 8						// 8
#define cPropertySelectionChangeEvent (jlong) 16				// 10
#define cPropertyTextChangeEvent (jlong) 32					   	// 20
#define cPropertyCaretChangeEvent (jlong) 64					// 40
#define cPropertyVisibleDataChangeEvent (jlong) 128				// 80
#define cPropertyChildChangeEvent (jlong) 256					// 100
#define cPropertyActiveDescendentChangeEvent (jlong) 512		// 200

#define cPropertyTableModelChangeEvent (jlong) 1024				// 400

/**
 ******************************************************
 *  optional AccessibleContext interfaces
 *
 * This version of the bridge reuses the accessibleValue
 * field in the AccessibleContextInfo struct to represent
 * additional optional interfaces that are supported by
 * the Java AccessibleContext.  This is backwardly compatable
 * because the old accessibleValue was set to the BOOL 
 * value TRUE (i.e., 1) if the AccessibleValue interface is 
 * supported.
 ******************************************************
 */

#define cAccessibleValueInterface (jlong) 1						// 1 << 1 (TRUE)
#define cAccessibleActionInterface (jlong) 2					// 1 << 2
#define cAccessibleComponentInterface (jlong) 4					// 1 << 3
#define cAccessibleSelectionInterface (jlong) 8					// 1 << 4
#define cAccessibleTableInterface (jlong) 16					// 1 << 5
#define cAccessibleTextInterface (jlong) 32						// 1 << 6


/**
 ******************************************************
 *  Accessibility information bundles
 ******************************************************
 */

typedef struct AccessBridgeVersionInfoTag {
	wchar_t VMversion[SHORT_STRING_SIZE];               // output of "java -version"
	wchar_t bridgeJavaClassVersion[SHORT_STRING_SIZE];  // version of the AccessBridge.class
	wchar_t bridgeJavaDLLVersion[SHORT_STRING_SIZE];    // version of JavaAccessBridge.dll
	wchar_t bridgeWinDLLVersion[SHORT_STRING_SIZE];     // version of WindowsAccessBridge.dll
} AccessBridgeVersionInfo;


typedef struct AccessibleContextInfoTag {
	wchar_t name[MAX_STRING_SIZE];			// the AccessibleName of the object
	wchar_t description[MAX_STRING_SIZE];	// the AccessibleDescription of the object

	wchar_t role[SHORT_STRING_SIZE];		// localized AccesibleRole string
	wchar_t role_en_US[SHORT_STRING_SIZE];	// AccesibleRole string in the en_US locale
	wchar_t states[SHORT_STRING_SIZE];		// localized AccesibleStateSet string (comma separated)
	wchar_t states_en_US[SHORT_STRING_SIZE]; // AccesibleStateSet string in the en_US locale (comma separated)

	jint indexInParent;					// index of object in parent
	jint childrenCount;					// # of children, if any

	jint x;								// screen coords in pixels
	jint y;								// "
	jint width;							// pixel width of object
	jint height;						// pixel height of object
									
	BOOL accessibleComponent;			// flags for various additional
	BOOL accessibleAction;				//  Java Accessibility interfaces
	BOOL accessibleSelection;			//  FALSE if this object doesn't
	BOOL accessibleText;				//  implement the additional interface
										//  in question

	// BOOL accessibleValue;            // old BOOL indicating whether AccessibleValue is supported
	BOOL accessibleInterfaces;			// new bitfield containing additional interface flags

} AccessibleContextInfo;



// AccessibleText packages
typedef struct AccessibleTextInfoTag {
	jint charCount;						// # of characters in this text object
	jint caretIndex;					// index of caret
	jint indexAtPoint;					// index at the passsed in point
} AccessibleTextInfo;

typedef struct AccessibleTextItemsInfoTag {
	wchar_t letter;
	wchar_t word[SHORT_STRING_SIZE];
	wchar_t sentence[MAX_STRING_SIZE];
} AccessibleTextItemsInfo;

typedef struct AccessibleTextSelectionInfoTag {
	jint selectionStartIndex;
	jint selectionEndIndex;
	wchar_t selectedText[MAX_STRING_SIZE];
} AccessibleTextSelectionInfo;

typedef struct AccessibleTextRectInfoTag {
	jint x;								// bounding rect of char at index
	jint y;								// "
	jint width;							// "
	jint height;						// "
} AccessibleTextRectInfo;

// standard attributes for text; note: tabstops are not supported
typedef struct AccessibleTextAttributesInfoTag {
	BOOL bold;
    BOOL italic;
    BOOL underline;
    BOOL strikethrough;
    BOOL superscript;
    BOOL subscript;

    wchar_t backgroundColor[SHORT_STRING_SIZE];
    wchar_t foregroundColor[SHORT_STRING_SIZE];
    wchar_t fontFamily[SHORT_STRING_SIZE];
    jint fontSize;

    jint alignment;
    jint bidiLevel;

    jfloat firstLineIndent;
    jfloat leftIndent;
    jfloat rightIndent;
    jfloat lineSpacing;
    jfloat spaceAbove;
    jfloat spaceBelow;

    wchar_t fullAttributesString[MAX_STRING_SIZE];
} AccessibleTextAttributesInfo;

/**
 ******************************************************
 *  IPC management typedefs
 ******************************************************
 */

#define cMemoryMappedNameSize	255

/**
 * sent by the WindowsDLL -> the memory-mapped file is setup
 *
 */
typedef struct MemoryMappedFileCreatedPackageTag {
	HWND bridgeWindow;		// redundant, but easier to get to here...
	char filename[cMemoryMappedNameSize];
} MemoryMappedFileCreatedPackage;




/**
 * sent when a new JavaVM attaches to the Bridge
 *
 */
typedef struct JavaVMCreatedPackageTag {
	HWND bridgeWindow;
	long vmID;
} JavaVMCreatedPackage;

/**
 * sent when a JavaVM detatches from the Bridge
 *
 */
typedef struct JavaVMDestroyedPackageTag {
	HWND bridgeWindow;
} JavaVMDestroyedPackage;

/**
 * sent when a new AT attaches to the Bridge
 *
 */
typedef struct WindowsATCreatedPackageTag {
	HWND bridgeWindow;
} WindowsATCreatedPackage;

/**
 * sent when an AT detatches from the Bridge
 *
 */
typedef struct WindowsATDestroyedPackageTag {
	HWND bridgeWindow;
} WindowsATDestroyedPackage;


/**
 * sent by JVM Bridges in response to a WindowsATCreate
 * message; saying "howdy, welcome to the neighborhood"
 *
 */
typedef struct JavaVMPresentNotificationPackageTag {
	HWND bridgeWindow;
	long vmID;
} JavaVMPresentNotificationPackage;

/**
 * sent by AT Bridges in response to a JavaVMCreate
 * message; saying "howdy, welcome to the neighborhood"
 *
 */
typedef struct WindowsATPresentNotificationPackageTag {
	HWND bridgeWindow;
} WindowsATPresentNotificationPackage;


/**
 ******************************************************
 *  Core packages
 ******************************************************
 */

typedef struct ReleaseJavaObjectPackageTag {
	long vmID;
	jobject object;
} ReleaseJavaObjectPackage; 

typedef struct GetAccessBridgeVersionPackageTag {
	long vmID;                    // can't get VM info w/out a VM!
    AccessBridgeVersionInfo rVersionInfo;
} GetAccessBridgeVersionPackage;

typedef struct IsSameObjectPackageTag {
	long vmID;
	jobject obj1;
	jobject obj2;
	jboolean rResult;
} IsSameObjectPackage; 

/**
 ******************************************************
 *  Windows packages
 ******************************************************
 */

typedef struct IsJavaWindowPackageTag {
	jint window;
	jboolean rResult;
} IsJavaWindowPackage; 


typedef struct GetAccessibleContextFromHWNDPackageTag {
	jint window;
	long rVMID;
	jobject rAccessibleContext;
} GetAccessibleContextFromHWNDPackage; 


/**
 ******************************************************
 *  AccessibleContext packages
 ******************************************************
 */

typedef struct GetAccessibleContextAtPackageTag {
	jint x;
	jint y;
	long vmID;
	jobject AccessibleContext;		// look within this AC
	jobject rAccessibleContext;
} GetAccessibleContextAtPackage; 

typedef struct GetAccessibleContextWithFocusPackageTag {
	long rVMID;
	jobject rAccessibleContext;
} GetAccessibleContextWithFocusPackage; 

typedef struct GetAccessibleContextInfoPackageTag {
	long vmID;
	jobject AccessibleContext;
	AccessibleContextInfo rAccessibleContextInfo;
} GetAccessibleContextInfoPackage; 

typedef struct GetAccessibleChildFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint childIndex;
	jobject rAccessibleContext;
} GetAccessibleChildFromContextPackage; 

typedef struct GetAccessibleParentFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
	jobject rAccessibleContext;
} GetAccessibleParentFromContextPackage; 

/**
 ******************************************************
 *  AccessibleTable packages
 ******************************************************
 */

#define MAX_TABLE_SELECTIONS 64

// table information
typedef struct AccessibleTableInfoTag {
	jobject caption;	// AccesibleContext
    jobject summary;	// AccessibleContext
	jint rowCount;
	jint columnCount;
	jobject accessibleContext;
	jobject accessibleTable;
} AccessibleTableInfo;

typedef struct GetAccessibleTableInfoPackageTag {
	long vmID;
	jobject accessibleContext;
	AccessibleTableInfo rTableInfo;
} GetAccessibleTableInfoPackage;

// table cell information
typedef struct AccessibleTableCellInfoTag {
    jobject  accessibleContext;
    jint     index;
    jint     row;
    jint     column;
    jint     rowExtent;
    jint     columnExtent;
    jboolean isSelected;
} AccessibleTableCellInfo;

typedef struct GetAccessibleTableCellInfoPackageTag {
	long vmID;
	jobject accessibleTable;
	jint row;
	jint column;
	AccessibleTableCellInfo rTableCellInfo;
} GetAccessibleTableCellInfoPackage; 

typedef struct GetAccessibleTableRowHeaderPackageTag {
	long vmID;
	jobject accessibleContext;
	AccessibleTableInfo rTableInfo;
} GetAccessibleTableRowHeaderPackage;

typedef struct GetAccessibleTableColumnHeaderPackageTag {
	long vmID;
	jobject accessibleContext;
	AccessibleTableInfo rTableInfo;
} GetAccessibleTableColumnHeaderPackage;

typedef struct GetAccessibleTableRowDescriptionPackageTag {
	long vmID;
	jobject accessibleContext;
	jint row;
	jobject rAccessibleContext;
} GetAccessibleTableRowDescriptionPackage;

typedef struct GetAccessibleTableColumnDescriptionPackageTag {
	long vmID;
	jobject accessibleContext;
	jint column;
	jobject rAccessibleContext;
} GetAccessibleTableColumnDescriptionPackage;

typedef struct GetAccessibleTableRowSelectionCountPackageTag {
	long vmID;
	jobject accessibleTable;
	jint rCount;
} GetAccessibleTableRowSelectionCountPackage;

typedef struct IsAccessibleTableRowSelectedPackageTag {
	long vmID;
	jobject accessibleTable;
	jint row;
    jboolean rResult;
} IsAccessibleTableRowSelectedPackage;

typedef struct GetAccessibleTableRowSelectionsPackageTag {
	long vmID;
	jobject accessibleTable;
	jint count;
    jint rSelections[MAX_TABLE_SELECTIONS];
} GetAccessibleTableRowSelectionsPackage;

typedef struct GetAccessibleTableColumnSelectionCountPackageTag {
	long vmID;
	jobject accessibleTable;
	jint rCount;
} GetAccessibleTableColumnSelectionCountPackage;

typedef struct IsAccessibleTableColumnSelectedPackageTag {
	long vmID;
	jobject accessibleTable;
	jint column;
    jboolean rResult;
} IsAccessibleTableColumnSelectedPackage;

typedef struct GetAccessibleTableColumnSelectionsPackageTag {
	long vmID;
	jobject accessibleTable;
	jint count;
    jint rSelections[MAX_TABLE_SELECTIONS];
} GetAccessibleTableColumnSelectionsPackage;


typedef struct GetAccessibleTableRowPackageTag {
	long vmID;
	jobject accessibleTable;
	jint index;
    jint rRow;
} GetAccessibleTableRowPackage;

typedef struct GetAccessibleTableColumnPackageTag {
	long vmID;
	jobject accessibleTable;
	jint index;
    jint rColumn;
} GetAccessibleTableColumnPackage;

typedef struct GetAccessibleTableIndexPackageTag {
	long vmID;
	jobject accessibleTable;
	jint row;
    jint column;
	jint rIndex;
} GetAccessibleTableIndexPackage;


/**
 ******************************************************
 *  AccessibleRelationSet packages
 ******************************************************
 */

#define MAX_RELATION_TARGETS 25
#define MAX_RELATIONS 5

typedef struct AccessibleRelationInfoTag {
	wchar_t key[SHORT_STRING_SIZE];
	jint targetCount;
	jobject targets[MAX_RELATION_TARGETS];  // AccessibleContexts
} AccessibleRelationInfo;

typedef struct AccessibleRelationSetInfoTag {
	jint relationCount;
	AccessibleRelationInfo relations[MAX_RELATIONS];
} AccessibleRelationSetInfo;

typedef struct GetAccessibleRelationSetPackageTag {
	long vmID;
	jobject accessibleContext;
	AccessibleRelationSetInfo rAccessibleRelationSetInfo;
} GetAccessibleRelationSetPackage;


/**
 ******************************************************
 *  AccessibleText packages
 ******************************************************
 */

typedef struct GetAccessibleTextInfoPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint x;
	jint y;
	AccessibleTextInfo rTextInfo;
} GetAccessibleTextInfoPackage;

typedef struct GetAccessibleTextItemsPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint index;
	AccessibleTextItemsInfo rTextItemsInfo;
} GetAccessibleTextItemsPackage;

typedef struct GetAccessibleTextSelectionInfoPackageTag {
	long vmID;
	jobject AccessibleContext;
	AccessibleTextSelectionInfo rTextSelectionItemsInfo;
} GetAccessibleTextSelectionInfoPackage;

typedef struct GetAccessibleTextAttributeInfoPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint index;
	AccessibleTextAttributesInfo rAttributeInfo;
} GetAccessibleTextAttributeInfoPackage;

typedef struct GetAccessibleTextRectInfoPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint index;
	AccessibleTextRectInfo rTextRectInfo;
} GetAccessibleTextRectInfoPackage;

typedef struct GetAccessibleTextLineBoundsPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint index;
	jint rLineStart;
	jint rLineEnd;
} GetAccessibleTextLineBoundsPackage;

typedef struct GetAccessibleTextRangePackageTag {
	long vmID;
	jobject AccessibleContext;
	jint start;
    jint end;
	wchar_t rText[MAX_STRING_SIZE];
} GetAccessibleTextRangePackage;


/**
 ******************************************************
 *  AccessibleValue packages
 ******************************************************
 */

typedef struct GetCurrentAccessibleValueFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
	wchar_t rValue[SHORT_STRING_SIZE];
} GetCurrentAccessibleValueFromContextPackage;

typedef struct GetMaximumAccessibleValueFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
	wchar_t rValue[SHORT_STRING_SIZE];
} GetMaximumAccessibleValueFromContextPackage;

typedef struct GetMinimumAccessibleValueFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
	wchar_t rValue[SHORT_STRING_SIZE];
} GetMinimumAccessibleValueFromContextPackage;


/**
 ******************************************************
 *  AccessibleSelection packages
 ******************************************************
 */

typedef struct AddAccessibleSelectionFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint index;
} AddAccessibleSelectionFromContextPackage;

typedef struct ClearAccessibleSelectionFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
} ClearAccessibleSelectionFromContextPackage;

typedef struct GetAccessibleSelectionFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
    jint index;
	jobject rAccessibleContext;
} GetAccessibleSelectionFromContextPackage;

typedef struct GetAccessibleSelectionCountFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint rCount;
} GetAccessibleSelectionCountFromContextPackage;

typedef struct IsAccessibleChildSelectedFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint index;
    jboolean rResult;
} IsAccessibleChildSelectedFromContextPackage;

typedef struct RemoveAccessibleSelectionFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
	jint index;
} RemoveAccessibleSelectionFromContextPackage;

typedef struct SelectAllAccessibleSelectionFromContextPackageTag {
	long vmID;
	jobject AccessibleContext;
} SelectAllAccessibleSelectionFromContextPackage;


/**
 ******************************************************
 *  Java Event Notification Registration packages
 ******************************************************
 */

typedef struct AddJavaEventNotificationPackageTag {
	jlong type;
	HWND DLLwindow;
} AddJavaEventNotificationPackage;

typedef struct RemoveJavaEventNotificationPackageTag {
	jlong type;
	HWND DLLwindow;
} RemoveJavaEventNotificationPackage;


/**
 ******************************************************
 *  Accessibility Event Notification Registration packages
 ******************************************************
 */

typedef struct AddAccessibilityEventNotificationPackageTag {
	jlong type;
	HWND DLLwindow;
} AddAccessibilityEventNotificationPackage;

typedef struct RemoveAccessibilityEventNotificationPackageTag {
	jlong type;
	HWND DLLwindow;
} RemoveAccessibilityEventNotificationPackage;


/**
 ******************************************************
 *  Accessibility Property Change Event packages
 ******************************************************
 */

typedef struct PropertyCaretChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
	jint oldPosition;
	jint newPosition;
} PropertyCaretChangePackage;

typedef struct PropertyDescriptionChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
	wchar_t oldDescription[SHORT_STRING_SIZE];
	wchar_t newDescription[SHORT_STRING_SIZE];
} PropertyDescriptionChangePackage;

typedef struct PropertyNameChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
	wchar_t oldName[SHORT_STRING_SIZE];
	wchar_t newName[SHORT_STRING_SIZE];
} PropertyNameChangePackage;

typedef struct PropertySelectionChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
} PropertySelectionChangePackage;

typedef struct PropertyStateChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
	wchar_t oldState[SHORT_STRING_SIZE];
	wchar_t newState[SHORT_STRING_SIZE];
} PropertyStateChangePackage;

typedef struct PropertyTextChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
} PropertyTextChangePackage;

typedef struct PropertyValueChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
	wchar_t oldValue[SHORT_STRING_SIZE];
	wchar_t newValue[SHORT_STRING_SIZE];
} PropertyValueChangePackage;

typedef struct PropertyVisibleDataChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
} PropertyVisibleDataChangePackage;

typedef struct PropertyChildChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
	jobject oldChildAccessibleContext;
	jobject newChildAccessibleContext;
} PropertyChildChangePackage;

typedef struct PropertyActiveDescendentChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
	jobject oldActiveDescendentAccessibleContext;
	jobject newActiveDescendentAccessibleContext;
} PropertyActiveDescendentChangePackage;


// String format for newValue is:
//	"type" one of "INSERT", "UPDATE" or "DELETE"
//	"firstRow"
//	"lastRow"
//	"firstColumn"
//	"lastColumn"
//
// oldValue is currently unused
//
typedef struct PropertyTableModelChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
	wchar_t oldValue[SHORT_STRING_SIZE];
	wchar_t newValue[SHORT_STRING_SIZE];
} PropertyTableModelChangePackage;


/**
 ******************************************************
 *  Property Change Event packages
 ******************************************************
 */

/*
typedef struct PropertyChangePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
	char propertyName[SHORT_STRING_SIZE];
	char oldValue[SHORT_STRING_SIZE];	// PropertyChangeEvent().getOldValue().toString()
	char newValue[SHORT_STRING_SIZE];	// PropertyChangeEvent().getNewValue().toString()
} PropertyChangePackage;
*/

/**
 ******************************************************
 *  Focus Event packages
 ******************************************************
 */

typedef struct FocusGainedPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
} FocusGainedPackage; 

typedef struct FocusLostPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
} FocusLostPackage; 


/**
 ******************************************************
 *  Caret Event packages
 ******************************************************
 */

typedef struct CaretUpdatePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
} CaretUpdatePackage; 


/**
 ******************************************************
 *  Mouse Event packages
 ******************************************************
 */

typedef struct MouseClickedPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
} MouseClickedPackage; 

typedef struct MouseEnteredPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
} MouseEnteredPackage;

typedef struct MouseExitedPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
} MouseExitedPackage;

typedef struct MousePressedPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
} MousePressedPackage;

typedef struct MouseReleasedPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
} MouseReleasedPackage;


/**
 ******************************************************
 *  Menu/PopupMenu Event packages
 ******************************************************
 */

typedef struct MenuCanceledPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
} MenuCanceledPackage;

typedef struct MenuDeselectedPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
} MenuDeselectedPackage;

typedef struct MenuSelectedPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
} MenuSelectedPackage;


typedef struct PopupMenuCanceledPackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
} PopupMenuCanceledPackage;

typedef struct PopupMenuWillBecomeInvisiblePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
} PopupMenuWillBecomeInvisiblePackage;

typedef struct PopupMenuWillBecomeVisiblePackageTag {
	long vmID;
	jobject Event;
	jobject AccessibleContextSource;
} PopupMenuWillBecomeVisiblePackage;



/**
 ******************************************************
 *  Wrapping up all of the packages
 ******************************************************
 */

/**
 *  What is the type of this package
 */
typedef enum PackageType {

	cMemoryMappedFileCreatedPackage = 0x11000,

		// many of these will go away...
	cJavaVMCreatedPackage = 0x10000,
	cJavaVMDestroyedPackage,
	cWindowsATCreatedPackage,
	cWindowsATDestroyedPackage,
	cJavaVMPresentNotificationPackage,
	cWindowsATPresentNotificationPackage,
	
	cReleaseJavaObjectPackage = 1,
    cGetAccessBridgeVersionPackage = 2,

	cGetAccessibleContextFromHWNDPackage = 0x10,
    cIsJavaWindowPackage,
	cIsSameObjectPackage,

    cGetAccessibleContextAtPackage = 0x100,
    cGetAccessibleContextWithFocusPackage,
    cGetAccessibleContextInfoPackage,
	cGetAccessibleChildFromContextPackage,
	cGetAccessibleParentFromContextPackage,

	cGetAccessibleTextInfoPackage = 0x200,
	cGetAccessibleTextItemsPackage,
	cGetAccessibleTextSelectionInfoPackage,
	cGetAccessibleTextAttributeInfoPackage,
	cGetAccessibleTextRectInfoPackage,
	cGetAccessibleTextLineBoundsPackage,
	cGetAccessibleTextRangePackage,

	cGetCurrentAccessibleValueFromContextPackage = 0x300,
	cGetMaximumAccessibleValueFromContextPackage,
	cGetMinimumAccessibleValueFromContextPackage,

	cAddAccessibleSelectionFromContextPackage = 0x400,
	cClearAccessibleSelectionFromContextPackage,
	cGetAccessibleSelectionFromContextPackage,
	cGetAccessibleSelectionCountFromContextPackage,
	cIsAccessibleChildSelectedFromContextPackage,
	cRemoveAccessibleSelectionFromContextPackage,
	cSelectAllAccessibleSelectionFromContextPackage,

	cAddJavaEventNotificationPackage = 0x900,
	cRemoveJavaEventNotificationPackage,
	cAddAccessibilityEventNotificationPackage,
	cRemoveAccessibilityEventNotificationPackage,

	cPropertyChangePackage = 0x1000,

	cFocusGainedPackage = 0x1010,
	cFocusLostPackage,

	cCaretUpdatePackage = 0x1020,

	cMouseClickedPackage = 0x1030,
	cMouseEnteredPackage,
	cMouseExitedPackage,
	cMousePressedPackage,
	cMouseReleasedPackage,

	cMenuCanceledPackage = 0x1040,
	cMenuDeselectedPackage,
	cMenuSelectedPackage,
	cPopupMenuCanceledPackage,
	cPopupMenuWillBecomeInvisiblePackage,
	cPopupMenuWillBecomeVisiblePackage,

   	cPropertyCaretChangePackage = 0x1100,
   	cPropertyDescriptionChangePackage,
   	cPropertyNameChangePackage,
   	cPropertySelectionChangePackage,
   	cPropertyStateChangePackage,
   	cPropertyTextChangePackage,
   	cPropertyValueChangePackage,
   	cPropertyVisibleDataChangePackage,
   	cPropertyChildChangePackage,
   	cPropertyActiveDescendentChangePackage,


	// AccessibleTable
	cGetAccessibleTableInfoPackage = 0x1200,
	cGetAccessibleTableCellInfoPackage,

	cGetAccessibleTableRowHeaderPackage,
	cGetAccessibleTableColumnHeaderPackage,

	cGetAccessibleTableRowDescriptionPackage,
    cGetAccessibleTableColumnDescriptionPackage,

	cGetAccessibleTableRowSelectionCountPackage,
    cIsAccessibleTableRowSelectedPackage,
    cGetAccessibleTableRowSelectionsPackage,

	cGetAccessibleTableColumnSelectionCountPackage,
    cIsAccessibleTableColumnSelectedPackage,
    cGetAccessibleTableColumnSelectionsPackage,

	cGetAccessibleTableRowPackage,
	cGetAccessibleTableColumnPackage,
	cGetAccessibleTableIndexPackage,

	cPropertyTableModelChangePackage,


	// AccessibleRelationSet
	cGetAccessibleRelationSetPackage = 0x1300


} PackageType;


/**
 *  Union of all package contents
 */
typedef union AllPackagesTag {

	// Initial Rendezvous packages
	MemoryMappedFileCreatedPackage memoryMappedFileCreatedPackage;

	JavaVMCreatedPackage javaVMCreatedPackage;
	JavaVMDestroyedPackage javaVMDestroyedPackage;
	WindowsATCreatedPackage windowsATCreatedPackage;
	WindowsATDestroyedPackage windowsATDestroyedPackage;
	JavaVMPresentNotificationPackage javaVMPresentNotificationPackage;
	WindowsATPresentNotificationPackage windowsATPresentNotificationPackage;

	// Core packages
	ReleaseJavaObjectPackage releaseJavaObject;
    GetAccessBridgeVersionPackage getAccessBridgeVersion;
	IsSameObjectPackage _isSamePackage;

	// Window packages
	GetAccessibleContextFromHWNDPackage getAccessibleContextFromHWND;

	// AccessibleContext packages
    GetAccessibleContextAtPackage getAccessibleContextAt;
    GetAccessibleContextWithFocusPackage getAccessibleContextWithFocus;
    GetAccessibleContextInfoPackage getAccessibleContextInfo;
	GetAccessibleChildFromContextPackage getAccessibleChildFromContext;
	GetAccessibleParentFromContextPackage getAccessibleParentFromContext;

	// AccessibleText packages
	GetAccessibleTextInfoPackage getAccessibleTextInfo;
	GetAccessibleTextItemsPackage getAccessibleTextItems;
	GetAccessibleTextSelectionInfoPackage getAccessibleTextSelectionInfo;
	GetAccessibleTextAttributeInfoPackage getAccessibleTextAttributeInfo;
	GetAccessibleTextRectInfoPackage getAccessibleTextRectInfo;
    GetAccessibleTextLineBoundsPackage getAccessibleTextLineBounds;
    GetAccessibleTextRangePackage getAccessibleTextRange;

	// AccessibleValue packages
    GetCurrentAccessibleValueFromContextPackage getCurrentAccessibleValueFromContext;
    GetMaximumAccessibleValueFromContextPackage getMaximumAccessibleValueFromContext;
    GetMinimumAccessibleValueFromContextPackage getMinimumAccessibleValueFromContext;

	// AccessibleSelection packages
    AddAccessibleSelectionFromContextPackage addAccessibleSelectionFromContext;
    ClearAccessibleSelectionFromContextPackage clearAccessibleSelectionFromContext;
    GetAccessibleSelectionFromContextPackage getAccessibleSelectionFromContext;
    GetAccessibleSelectionCountFromContextPackage getAccessibleSelectionCountFromContext;
    IsAccessibleChildSelectedFromContextPackage isAccessibleChildSelectedFromContext;
    RemoveAccessibleSelectionFromContextPackage removeAccessibleSelectionFromContext;
    SelectAllAccessibleSelectionFromContextPackage selectAllAccessibleSelectionFromContext;

	// Event Notification Registration packages
	AddJavaEventNotificationPackage addJavaEventNotification;
	RemoveJavaEventNotificationPackage removeJavaEventNotification;
	AddAccessibilityEventNotificationPackage addAccessibilityEventNotification;
	RemoveAccessibilityEventNotificationPackage removeAccessibilityEventNotification;

	// Event contents packages
//	PropertyChangePackage propertyChange;
    PropertyCaretChangePackage propertyCaretChangePackage;
    PropertyDescriptionChangePackage propertyDescriptionChangePackage;
    PropertyNameChangePackage propertyNameChangePackage;
    PropertySelectionChangePackage propertySelectionChangePackage;
    PropertyStateChangePackage propertyStateChangePackage;
    PropertyTextChangePackage propertyTextChangePackage;
    PropertyValueChangePackage propertyValueChangePackage;
    PropertyVisibleDataChangePackage propertyVisibleDataChangePackage;
    PropertyChildChangePackage propertyChildChangePackage;
    PropertyActiveDescendentChangePackage propertyActiveDescendentChangePackage;

	PropertyTableModelChangePackage propertyTableModelChangePackage;

	FocusGainedPackage focusGained;
	FocusLostPackage focusLost;

	CaretUpdatePackage caretUpdate;

	MouseClickedPackage mouseClicked;
	MouseEnteredPackage mouseEntered;
	MouseExitedPackage mouseExited;
	MousePressedPackage mousePressed;
	MouseReleasedPackage mouseReleased;

	MenuCanceledPackage menuCanceled;
	MenuDeselectedPackage menuDeselected;
	MenuSelectedPackage menuSelected;
	PopupMenuCanceledPackage popupMenuCanceled;
	PopupMenuWillBecomeInvisiblePackage popupMenuWillBecomeInvisible;
	PopupMenuWillBecomeVisiblePackage popupMenuWillBecomeVisible;


} AllPackages;


/**
 *  Union of all Java-initiated package contents
 */
typedef union JavaInitiatedPackagesTag {

	// Initial Rendezvous packages
	JavaVMCreatedPackage javaVMCreatedPackage;
	JavaVMDestroyedPackage javaVMDestroyedPackage;
	JavaVMPresentNotificationPackage javaVMPresentNotificationPackage;

	// Event contents packages
//	PropertyChangePackage propertyChange;
    PropertyCaretChangePackage propertyCaretChangePackage;
    PropertyDescriptionChangePackage propertyDescriptionChangePackage;
    PropertyNameChangePackage propertyNameChangePackage;
    PropertySelectionChangePackage propertySelectionChangePackage;
    PropertyStateChangePackage propertyStateChangePackage;
    PropertyTextChangePackage propertyTextChangePackage;
    PropertyValueChangePackage propertyValueChangePackage;
    PropertyVisibleDataChangePackage propertyVisibleDataChangePackage;
    PropertyChildChangePackage propertyChildChangePackage;
    PropertyActiveDescendentChangePackage propertyActiveDescendentChangePackage;

	PropertyTableModelChangePackage propertyTableModelChangePackage;

	FocusGainedPackage focusGained;
	FocusLostPackage focusLost;

	CaretUpdatePackage caretUpdate;

	MouseClickedPackage mouseClicked;
	MouseEnteredPackage mouseEntered;
	MouseExitedPackage mouseExited;
	MousePressedPackage mousePressed;
	MouseReleasedPackage mouseReleased;

	MenuCanceledPackage menuCanceled;
	MenuDeselectedPackage menuDeselected;
	MenuSelectedPackage menuSelected;
	PopupMenuCanceledPackage popupMenuCanceled;
	PopupMenuWillBecomeInvisiblePackage popupMenuWillBecomeInvisible;
	PopupMenuWillBecomeVisiblePackage popupMenuWillBecomeVisible;

} JavaInitiatedPackages;


/**
 *  Union of all Windows-initiated package contents
 */
typedef union WindowsInitiatedPackagesTag {

	// Initial Rendezvous packages
	MemoryMappedFileCreatedPackage memoryMappedFileCreatedPackage;

	WindowsATCreatedPackage windowsATCreatedPackage;
	WindowsATDestroyedPackage windowsATDestroyedPackage;
	WindowsATPresentNotificationPackage windowsATPresentNotificationPackage;

	// Core packages
	ReleaseJavaObjectPackage releaseJavaObject;
    GetAccessBridgeVersionPackage getAccessBridgeVersion;

	// Window packages
	GetAccessibleContextFromHWNDPackage getAccessibleContextFromHWND;

	// AccessibleContext packages
    GetAccessibleContextAtPackage getAccessibleContextAt;
    GetAccessibleContextWithFocusPackage getAccessibleContextWithFocus;
    GetAccessibleContextInfoPackage getAccessibleContextInfo;
	GetAccessibleChildFromContextPackage getAccessibleChildFromContext;
	GetAccessibleParentFromContextPackage getAccessibleParentFromContext;

	// AccessibleText packages
	GetAccessibleTextInfoPackage getAccessibleTextInfo;
	GetAccessibleTextItemsPackage getAccessibleTextItems;
	GetAccessibleTextSelectionInfoPackage getAccessibleTextSelectionInfo;
	GetAccessibleTextAttributeInfoPackage getAccessibleTextAttributeInfo;
	GetAccessibleTextRectInfoPackage getAccessibleTextRectInfo;
    GetAccessibleTextLineBoundsPackage getAccessibleTextLineBounds;
    GetAccessibleTextRangePackage getAccessibleTextRange;

	// AccessibleValue packages
    GetCurrentAccessibleValueFromContextPackage getCurrentAccessibleValueFromContext;
    GetMaximumAccessibleValueFromContextPackage getMaximumAccessibleValueFromContext;
    GetMinimumAccessibleValueFromContextPackage getMinimumAccessibleValueFromContext;

	// AccessibleSelection packages
    AddAccessibleSelectionFromContextPackage addAccessibleSelectionFromContext;
    ClearAccessibleSelectionFromContextPackage clearAccessibleSelectionFromContext;
    GetAccessibleSelectionFromContextPackage getAccessibleSelectionFromContext;
    GetAccessibleSelectionCountFromContextPackage getAccessibleSelectionCountFromContext;
    IsAccessibleChildSelectedFromContextPackage isAccessibleChildSelectedFromContext;
    RemoveAccessibleSelectionFromContextPackage removeAccessibleSelectionFromContext;
    SelectAllAccessibleSelectionFromContextPackage selectAllAccessibleSelectionFromContext;

    // Event Notification Registration packages
	AddJavaEventNotificationPackage addJavaEventNotification;
	RemoveJavaEventNotificationPackage removeJavaEventNotification;
	AddAccessibilityEventNotificationPackage addAccessibilityEventNotification;
	RemoveAccessibilityEventNotificationPackage removeAccessibilityEventNotification;

		// AccessibleTable
	GetAccessibleTableInfoPackage _getAccessibleTableInfo;
	GetAccessibleTableCellInfoPackage _getAccessibleTableCellInfo;

	GetAccessibleTableRowHeaderPackage _getAccessibleTableRowHeader;
	GetAccessibleTableColumnHeaderPackage _getAccessibleTableColumnHeader;

	GetAccessibleTableRowDescriptionPackage _getAccessibleTableRowDescription;
	GetAccessibleTableColumnDescriptionPackage _getAccessibleTableColumnDescription;

	GetAccessibleTableRowSelectionCountPackage _getAccessibleTableRowSelectionCount;
    IsAccessibleTableRowSelectedPackage _isAccessibleTableRowSelected;
    GetAccessibleTableRowSelectionsPackage _getAccessibleTableRowSelections;

    GetAccessibleTableColumnSelectionCountPackage _getAccessibleTableColumnSelectionCount;
    IsAccessibleTableColumnSelectedPackage _isAccessibleTableColumnSelected;
    GetAccessibleTableColumnSelectionsPackage _getAccessibleTableColumnSelections;

	GetAccessibleTableRowPackage _getAccessibleTableRow;
	GetAccessibleTableColumnPackage _getAccessibleTableColumn;
	GetAccessibleTableIndexPackage _getAccessibleTableIndex;

	// AccessibleRelationSet
	GetAccessibleRelationSetPackage _getAccessibleRelationSet;

	IsSameObjectPackage _isSameObject;

} WindowsInitiatedPackages;


#ifdef __cplusplus
}
#endif

#endif
